
/*
 * Copyright (C) 2009 Freescale Semiconductor, Inc. All rights reserved.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <pci.h>
#include <asm/processor.h>
#include <asm/mmu.h>
#include <asm/cache.h>
#include <asm/immap_85xx.h>
#include <asm/immap_fsl_pci.h>
#include <asm/fsl_ddr_sdram.h>
#include <asm/io.h>
#include <asm/fsl_law.h>
#include <miiphy.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <tsec.h>
#include <vsc7385.h>


#if defined(CONFIG_DDR_ECC) && !defined(CONFIG_ECC_INIT_VIA_DDRCONTROLLER)
extern void ddr_enable_ecc(unsigned int dram_size);
#endif

#define GPIO_DIR	0xffff0000
#define BOARD_PERI_RST	0x023f0000
#define USB_RST		0x04000000
#define PUPLD2_SET	0x00100000
#define CNPLD_SET	0x00200000

#define SYSCLK_MASK	0x00200000
#define BOARDREV_MASK	0x10100000
#define BOARDREV_A	0x10000000
#define BOARDREV_B	0x10100000
#define BOARDREV_C	0x00100000

#define SYSCLK_66	66666666
#define SYSCLK_50	50000000
#define SYSCLK_100	100000000
DECLARE_GLOBAL_DATA_PTR;

phys_size_t  fixed_sdram(void);
void local_bus_init(void);
void idt89hpes16t4ag2_init(void);

unsigned long get_board_sys_clk(ulong dummy)
{
	volatile ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
	u32 val, sysclk, temp;

	return SYSCLK_100; /* pu-132 */

	val = pgpio->gpdat;
	sysclk = val & SYSCLK_MASK;
	temp = val & BOARDREV_MASK;
	if (temp == BOARDREV_C)
		if(sysclk == 0)
			return SYSCLK_66;
		else
			return SYSCLK_100;
	else if (temp == BOARDREV_B)
		if(sysclk == 0)
			return SYSCLK_66;
		else
			return SYSCLK_50;
	else if (temp == BOARDREV_A)
		return SYSCLK_50;
}

unsigned long get_board_ddr_size(ulong dummy)
{
	volatile ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
	u32 val, temp;
	unsigned int ddr_size;

	val = pgpio->gpdat;
	temp = val & BOARDREV_MASK;

	if(gd->cpu->soc_ver == SVR_P1020 ||
	   gd->cpu->soc_ver == SVR_P1020_E)
		ddr_size = 256 * 1024 * 1024;
	if(gd->cpu->soc_ver == SVR_P2020 ||
	   gd->cpu->soc_ver == SVR_P2020_E) {
		if(temp == 0)
			ddr_size = 512 * 1024 * 1024;
		else
			ddr_size = 1024 * 1024 * 1024;
	}
/*	return ddr_size;	pu-132 */
	return 512 * 1024 * 1024; /* pu-132 */
}

int board_early_init_f (void)
{
#ifdef CONFIG_MMC
	volatile ccsr_gur_t *gur = (void *)(CONFIG_SYS_MPC85xx_GUTS_ADDR);

	setbits_be32(&gur->pmuxcr,
			(MPC85xx_PMUXCR_SDHC_CD |
			 MPC85xx_PMUXCR_SDHC_WP));
#endif

	local_bus_init();

	return 0;
}

int checkboard (void)
{
	u32 val, temp;
	volatile ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
	char board_rev = 0;

/* Bringing the following peripherals out of reset via GPIOs
 * 0= reset and 1= out of reset
 * GPIO12 - Reset to Ethernet Switch
 * GPIO13 - Reset to SLIC/SLAC devices
 * GPIO14 - Reset to SGMII_PHY_N
 * GPIO15 - Reset to PCIe slots
 * GPIO6  - Reset to RGMII PHY
 * GPIO5 - Reset to USB3300 devices 1= reset and 0= out of reset
 */
	setbits_be32(&pgpio->gpdat, PUPLD2_SET | CNPLD_SET);

	val = pgpio->gpdir;
	val |=  GPIO_DIR;
	setbits_be32(&pgpio->gpdir, GPIO_DIR);

	val = pgpio->gpdat;
	temp = val & BOARDREV_MASK;
	if (temp == BOARDREV_C)
		board_rev = 'C';
	else if (temp == BOARDREV_B)
		board_rev = 'B';
	else if (temp == BOARDREV_A)
		board_rev = 'A';

	board_rev = 'C'; /* pu-132 */

/*	if (board_rev == 0)
		printf ("Unknown P2020RDB board \n");
	else
		printf ("Board: %sRDB Rev%c, System ID: 0x%02x, "
			"System Version: 0x%02x\n", gd->cpu->name,
							board_rev, 0, 0);
*/
	printf ("Board: ");
#ifdef BOARD_CPU448
	printf ("CPU-448\n");
#endif
#ifdef BOARD_CPU453
	printf ("CPU-453\n");
#endif

	clrsetbits_be32(&pgpio->gpdat, USB_RST, BOARD_PERI_RST);

	return 0;
}

phys_size_t initdram(int board_type)
{
	phys_size_t dram_size = 0;

	puts("Initializing....");

#if defined(CONFIG_SDCARD_U_BOOT) || defined(CONFIG_NAND_U_BOOT)
	return CONFIG_SYS_SDRAM_SIZE;
#endif

#ifdef CONFIG_DDR_SPD
	dram_size = fsl_ddr_sdram();
#else
	dram_size = fixed_sdram();
	set_ddr_laws(0, dram_size, LAW_TRGT_IF_DDR_1);
#endif
	dram_size = setup_ddr_tlbs(dram_size / 0x100000);
	dram_size *= 0x100000;

#if defined(CONFIG_DDR_ECC) && !defined(CONFIG_ECC_INIT_VIA_DDRCONTROLLER)
	/*
	 * Initialize and enable DDR ECC.
	 */
	ddr_enable_ecc(dram_size);
#endif
	puts("    DDR: ");
	return dram_size;
}

#if !defined(CONFIG_SPD)
/*
 * Fixed sdram init -- doesn't use serial presence detect.
 */

phys_size_t fixed_sdram (void)
{

	volatile ccsr_ddr_t *ddr= (ccsr_ddr_t *)CONFIG_SYS_MPC85xx_DDR_ADDR;
	int dbw;
	u32 val, temp;
	volatile ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
	sys_info_t sysinfo;
	char board_rev = 0;
#if defined(CONFIG_ECC_INIT_VIA_DDRCONTROLLER)
	int d_init;
#endif
	ddr->cs0_bnds = CONFIG_SYS_DDR_CS0_BNDS;
	ddr->cs0_config = CONFIG_SYS_DDR_CS0_CONFIG;
	ddr->cs0_config_2 = CONFIG_SYS_DDR_CS0_CONFIG_2;
	ddr->sdram_data_init = CONFIG_SYS_DDR_DATA_INIT;

	/* On P2020/P1020 RDB boards DDR size varies as follows:
	 * REV A board (512MB P2020 and 256MB P1020)
	 * REV B board (1GB P2020 and 256MB P1020)
	 * FIXME:: must also program cs0_bnds register accordingly.
	 * currently CSO_BNDS is programmed for 1G.
	 */
	val = pgpio->gpdat;
	temp = val & BOARDREV_MASK;
	get_sys_info(&sysinfo);

	if (temp == BOARDREV_C)
		board_rev = 'C';
	else if (temp == BOARDREV_B)
		board_rev = 'B';
	else if (temp == BOARDREV_A)
		board_rev = 'A';

	board_rev = 'C'; /* pu-132 */

	printf("Configuring DDR freq%d\n", (int)sysinfo.freqDDRBus);
	if(temp == BOARDREV_A) {
		/* Rev A board*/
		ddr->timing_cfg_3 = CONFIG_SYS_DDR_TIMING_3_REVA;
		ddr->timing_cfg_0 = CONFIG_SYS_DDR_TIMING_0_REVA;
		ddr->timing_cfg_1 = CONFIG_SYS_DDR_TIMING_1_REVA;
		ddr->timing_cfg_2 = CONFIG_SYS_DDR_TIMING_2_REVA;
		ddr->timing_cfg_4 = CONFIG_SYS_DDR_TIMING_4_REVA;
		ddr->timing_cfg_5 = CONFIG_SYS_DDR_TIMING_5_REVA;
		ddr->sdram_mode = CONFIG_SYS_DDR_MODE_1_REVA;
		ddr->sdram_mode_2 = CONFIG_SYS_DDR_MODE_2_REVA;
		ddr->sdram_interval = CONFIG_SYS_DDR_INTERVAL_REVA;
		ddr->sdram_clk_cntl = CONFIG_SYS_DDR_CLK_CTRL_REVA;
		ddr->sdram_cfg = CONFIG_SYS_DDR_CONTROL_REVA;
		ddr->sdram_cfg_2 = CONFIG_SYS_DDR_CONTROL2_REVA;
	}
	else if(sysinfo.freqDDRBus <= 400000000) {
		/* RevB and RevC boards */
		ddr->timing_cfg_3 = CONFIG_SYS_DDR_TIMING_3_400_REVB;
		ddr->timing_cfg_0 = CONFIG_SYS_DDR_TIMING_0_400_REVB;
		ddr->timing_cfg_1 = CONFIG_SYS_DDR_TIMING_1_400_REVB;
		ddr->timing_cfg_2 = CONFIG_SYS_DDR_TIMING_2_400_REVB;
		ddr->timing_cfg_4 = CONFIG_SYS_DDR_TIMING_4_400_REVB;
		ddr->timing_cfg_5 = CONFIG_SYS_DDR_TIMING_5_400_REVB;
		ddr->sdram_mode = CONFIG_SYS_DDR_MODE_1_400_REVB;
		ddr->sdram_mode_2 = CONFIG_SYS_DDR_MODE_2_400_REVB;
		ddr->sdram_interval = CONFIG_SYS_DDR_INTERVAL_400_REVB;
		ddr->sdram_clk_cntl = CONFIG_SYS_DDR_CLK_CTRL_400_REVB;
		ddr->sdram_cfg = CONFIG_SYS_DDR_CONTROL_400_REVB;
		ddr->sdram_cfg_2 = CONFIG_SYS_DDR_CONTROL2_400_REVB;
	}
	else if(sysinfo.freqDDRBus <= 534000000) {
		ddr->timing_cfg_3 = CONFIG_SYS_DDR_TIMING_3_533_REVB;
		ddr->timing_cfg_0 = CONFIG_SYS_DDR_TIMING_0_533_REVB;
		ddr->timing_cfg_1 = CONFIG_SYS_DDR_TIMING_1_533_REVB;
		ddr->timing_cfg_2 = CONFIG_SYS_DDR_TIMING_2_533_REVB;
		ddr->timing_cfg_4 = CONFIG_SYS_DDR_TIMING_4_533_REVB;
		ddr->timing_cfg_5 = CONFIG_SYS_DDR_TIMING_5_533_REVB;
		ddr->sdram_mode = CONFIG_SYS_DDR_MODE_1_533_REVB;
		ddr->sdram_mode_2 = CONFIG_SYS_DDR_MODE_2_533_REVB;
		ddr->sdram_interval = CONFIG_SYS_DDR_INTERVAL_533_REVB;
		ddr->sdram_clk_cntl = CONFIG_SYS_DDR_CLK_CTRL_533_REVB;
		ddr->sdram_cfg = CONFIG_SYS_DDR_CONTROL_533_REVB;
		ddr->sdram_cfg_2 = CONFIG_SYS_DDR_CONTROL2_533_REVB;
	}
	else if(sysinfo.freqDDRBus <= 667000000) {
		ddr->timing_cfg_3 = CONFIG_SYS_DDR_TIMING_3_667_REVB;
		ddr->timing_cfg_0 = CONFIG_SYS_DDR_TIMING_0_667_REVB;
		ddr->timing_cfg_1 = CONFIG_SYS_DDR_TIMING_1_667_REVB;
		ddr->timing_cfg_2 = CONFIG_SYS_DDR_TIMING_2_667_REVB;
		ddr->timing_cfg_4 = CONFIG_SYS_DDR_TIMING_4_667_REVB;
		ddr->timing_cfg_5 = CONFIG_SYS_DDR_TIMING_5_667_REVB;
		ddr->sdram_mode = CONFIG_SYS_DDR_MODE_1_667_REVB;
		ddr->sdram_mode_2 = CONFIG_SYS_DDR_MODE_2_667_REVB;
		ddr->sdram_interval = CONFIG_SYS_DDR_INTERVAL_667_REVB;
		ddr->sdram_clk_cntl = CONFIG_SYS_DDR_CLK_CTRL_667_REVB;
		ddr->sdram_cfg = CONFIG_SYS_DDR_CONTROL_667_REVB;
		ddr->sdram_cfg_2 = CONFIG_SYS_DDR_CONTROL2_667_REVB;
	}
	else if(sysinfo.freqDDRBus <= 800000000) {
		ddr->timing_cfg_3 = CONFIG_SYS_DDR_TIMING_3_800_REVB;
		ddr->timing_cfg_0 = CONFIG_SYS_DDR_TIMING_0_800_REVB;
		ddr->timing_cfg_1 = CONFIG_SYS_DDR_TIMING_1_800_REVB;
		ddr->timing_cfg_2 = CONFIG_SYS_DDR_TIMING_2_800_REVB;
		ddr->timing_cfg_4 = CONFIG_SYS_DDR_TIMING_4_800_REVB;
		ddr->timing_cfg_5 = CONFIG_SYS_DDR_TIMING_5_800_REVB;
		ddr->sdram_mode = CONFIG_SYS_DDR_MODE_1_800_REVB;
		ddr->sdram_mode_2 = CONFIG_SYS_DDR_MODE_2_800_REVB;
		ddr->sdram_interval = CONFIG_SYS_DDR_INTERVAL_800_REVB;
		ddr->sdram_clk_cntl = CONFIG_SYS_DDR_CLK_CTRL_800_REVB;
		ddr->sdram_cfg = CONFIG_SYS_DDR_CONTROL_800_REVB;
		ddr->sdram_cfg_2 = CONFIG_SYS_DDR_CONTROL2_800_REVB;
	}
 
	dbw = gd->cpu->ddr_data_width;
	if(dbw == 32) {
		/* need to check if this is required for P1020/P2020 also */
		printf("fixing DDR2 errata for 32bit\n");
		u32 temp = in_be32(&ddr->debug_1);
		out_be32(&ddr->debug_1, temp | 1);
	}

#if defined (CONFIG_DDR_ECC)
	ddr->err_int_en = CONFIG_SYS_DDR_ERR_INT_EN;
	ddr->err_disable = CONFIG_SYS_DDR_ERR_DIS;
	ddr->err_sbe = CONFIG_SYS_DDR_SBE;
#endif
	asm("sync;isync");

	udelay(500);

	if(dbw == 32)
		ddr->sdram_cfg |= 0x00080000;
	ddr->sdram_cfg |= SDRAM_CFG_MEM_EN;

#if defined(CONFIG_ECC_INIT_VIA_DDRCONTROLLER)
	d_init = 1;
	debug("DDR - 1st controller: memory initializing\n");
	/*
	 * Poll until memory is initialized.
	 * 512 Meg at 400 might hit this 200 times or so.
	 */
	while ((ddr->sdram_cfg_2 & (d_init << 4)) != 0) {
		udelay(1000);
	}
	debug("DDR: memory initialized\n\n");
	asm("sync; isync");
	udelay(500);
#endif

	return CONFIG_SYS_SDRAM_SIZE;
}

#endif

#ifdef CONFIG_PCIE1
static struct pci_controller pcie1_hose;
#endif

#ifdef CONFIG_PCIE2
static struct pci_controller pcie2_hose;
#endif

extern int fsl_pci_setup_inbound_windows(struct pci_region *r);
extern void fsl_pci_init(struct pci_controller *hose);

int first_free_busno=0;
#ifdef CONFIG_PCI
void pci_init_board(void)
{
	volatile ccsr_gur_t *gur = (void *)(CONFIG_SYS_MPC85xx_GUTS_ADDR);
	uint devdisr = gur->devdisr;
	uint io_sel = (gur->pordevsr & MPC85xx_PORDEVSR_IO_SEL) >> 19;
	uint host_agent = (gur->porbmsr & MPC85xx_PORBMSR_HA) >> 16;

	debug ("   pci_init_board: devdisr=%x, io_sel=%x, host_agent=%x\n",
			devdisr, io_sel, host_agent);

	if (!(gur->pordevsr & MPC85xx_PORDEVSR_SGMII2_DIS))
		printf ("    eTSEC2 is in sgmii mode.\n");

#ifdef CONFIG_PCIE2
	{
		volatile ccsr_fsl_pci_t *pci = (ccsr_fsl_pci_t *) CONFIG_SYS_PCIE2_ADDR;
		struct pci_controller *hose = &pcie2_hose;
		int pcie_ep = (host_agent == 2) || (host_agent == 4) ||
			(host_agent == 6) || (host_agent == 0);
		int pcie_configured  = (io_sel == 0xE);
		struct pci_region *r = hose->regions;

		if (pcie_configured && !(devdisr & MPC85xx_DEVDISR_PCIE)){
			printf ("\n    PCIE2 connected to Slot 1 as %s (base address %x)",
					pcie_ep ? "End Point" : "Root Complex",
					(uint)pci);
			if (pci->pme_msg_det) {
				pci->pme_msg_det = 0xffffffff;
				debug (" with errors.  Clearing.  Now 0x%08x",pci->pme_msg_det);
			}
			printf ("\n");

			/* inbound */
			r += fsl_pci_setup_inbound_windows(r);

			/* outbound memory */
			pci_set_region(r++,
					CONFIG_SYS_PCIE2_MEM_BUS,
					CONFIG_SYS_PCIE2_MEM_PHYS,
					CONFIG_SYS_PCIE2_MEM_SIZE,
					PCI_REGION_MEM);

			/* outbound io */
			pci_set_region(r++,
					CONFIG_SYS_PCIE2_IO_BUS,
					CONFIG_SYS_PCIE2_IO_PHYS,
					CONFIG_SYS_PCIE2_IO_SIZE,
					PCI_REGION_IO);

			hose->region_count = r - hose->regions;
			hose->first_busno=first_free_busno;
			pci_setup_indirect(hose, (int) &pci->cfg_addr, (int) &pci->cfg_data);

			fsl_pci_init(hose);
			first_free_busno=hose->last_busno+1;
			printf ("    PCIE2 on bus %02x - %02x\n",
					hose->first_busno,hose->last_busno);

		} else {
			printf ("    PCIE2: disabled\n");
		}

	}
#else
	gur->devdisr |= MPC85xx_DEVDISR_PCIE2; /* disable */
#endif
#ifdef CONFIG_PCIE1
	{
		volatile ccsr_fsl_pci_t *pci = (ccsr_fsl_pci_t *) CONFIG_SYS_PCIE1_ADDR;
		struct pci_controller *hose = &pcie1_hose;
		int pcie_ep = (host_agent <= 1) || (host_agent == 4) ||
			(host_agent == 5);
		int pcie_configured  = (io_sel == 0xE) ||(io_sel == 6);
		struct pci_region *r = hose->regions;

		if (pcie_configured && !(devdisr & MPC85xx_DEVDISR_PCIE)){
			printf ("\n    PCIE1 connected to Slot 2 as %s (base address %x)",
					pcie_ep ? "End Point" : "Root Complex",
					(uint)pci);
			if (pci->pme_msg_det) {
				pci->pme_msg_det = 0xffffffff;
				debug (" with errors.  Clearing.  Now 0x%08x",pci->pme_msg_det);
			}
			printf ("\n");

			/* inbound */
			r += fsl_pci_setup_inbound_windows(r);

			/* outbound memory */
			pci_set_region(r++,
					CONFIG_SYS_PCIE1_MEM_BUS,
					CONFIG_SYS_PCIE1_MEM_PHYS,
					CONFIG_SYS_PCIE1_MEM_SIZE,
					PCI_REGION_MEM);

			/* outbound io */
			pci_set_region(r++,
					CONFIG_SYS_PCIE1_IO_BUS,
					CONFIG_SYS_PCIE1_IO_PHYS,
					CONFIG_SYS_PCIE1_IO_SIZE,
					PCI_REGION_IO);

			hose->region_count = r - hose->regions;
			hose->first_busno=first_free_busno;

			pci_setup_indirect(hose, (int) &pci->cfg_addr, (int) &pci->cfg_data);

			fsl_pci_init(hose);

			idt89hpes16t4ag2_init();

			udelay(100000);

			fsl_pci_init(hose);

			first_free_busno=hose->last_busno+1;
			printf("    PCIE1 on bus %02x - %02x\n",
					hose->first_busno,hose->last_busno);

		} else {
			printf ("    PCIE1: disabled\n");
		}

	}
#else
	gur->devdisr |= MPC85xx_DEVDISR_PCIE; /* disable */
#endif
}
#endif

int board_early_init_r(void)
{
	const unsigned int flashbase = CONFIG_SYS_FLASH_BASE;
	const u8 flash_esel = 2;

	/*
	 * Remap Boot flash region to caching-inhibited
	 * so that flash can be erased properly.
	 */

	/* Flush d-cache and invalidate i-cache of any FLASH data */
	flush_dcache();
	invalidate_icache();

	/* invalidate existing TLB entry for flash */
	disable_tlb(flash_esel);

	set_tlb(1, flashbase, CONFIG_SYS_FLASH_BASE_PHYS,	/* tlb, epn, rpn */
			MAS3_SX|MAS3_SW|MAS3_SR, MAS2_I|MAS2_G,	/* perms, wimge */
			0, flash_esel, BOOKE_PAGESZ_16M, 1);	/* ts, esel, tsize, iprot */
	return 0;
}


#ifdef CONFIG_TSEC_ENET
int board_eth_init(bd_t *bis)
{
	struct tsec_info_struct tsec_info[4];
#ifdef CONFIG_TSEC3
	volatile ccsr_gur_t *gur = (void *)(CONFIG_SYS_MPC85xx_GUTS_ADDR);
#endif
	int num = 0;

#ifdef CONFIG_TSEC1
	SET_STD_TSEC_INFO(tsec_info[num], 1);
	num++;
#endif
#ifdef CONFIG_TSEC2
	SET_STD_TSEC_INFO(tsec_info[num], 2);
	num++;
#endif
#ifdef CONFIG_TSEC3
	SET_STD_TSEC_INFO(tsec_info[num], 3);
	if (!(gur->pordevsr & MPC85xx_PORDEVSR_SGMII3_DIS))
		tsec_info[num].flags |= TSEC_SGMII;
	num++;
#endif

	if (!num) {
		printf("No TSECs initialized\n");

		return 0;
	}

	tsec_eth_init(bis, tsec_info, num);

	return 0;
}
#endif

/*
 * Miscellaneous late-boot configurations
 *
 * If a VSC7385 microcode image is present, then upload it.
*/
int misc_init_r(void)
{
	int rc = 0;


#ifdef CONFIG_VSC7385_IMAGE
	printf(" uploading VSC7385 microcode.from %x\n", CONFIG_VSC7385_IMAGE);
	if (vsc7385_upload_firmware((void *) CONFIG_VSC7385_IMAGE,
					CONFIG_VSC7385_IMAGE_SIZE)) {
		puts("Failure uploading VSC7385 microcode.\n");
		rc = 1;
	}
#endif

	return rc;
}

#if defined(CONFIG_OF_BOARD_SETUP)
extern void ft_fsl_pci_setup(void *blob, const char *pci_alias,
				struct pci_controller *hose);

void ft_board_setup(void *blob, bd_t *bd)
{
	ulong base, size;

	ft_cpu_setup(blob, bd);

	base = getenv_bootm_low();
	size = getenv_bootm_size();

	fdt_fixup_memory(blob, (u64)base, (u64)size);

#ifdef CONFIG_PCIE2
	ft_fsl_pci_setup(blob, "pci1", &pcie2_hose);
#endif
#ifdef CONFIG_PCIE1
	ft_fsl_pci_setup(blob, "pci2", &pcie1_hose);
#endif
}
#endif

#ifdef CONFIG_MP
extern void cpu_mp_lmb_reserve(struct lmb *lmb);

void board_lmb_reserve(struct lmb *lmb)
{
	cpu_mp_lmb_reserve(lmb);
}
#endif

#if defined(CONFIG_SYS_DRAM_TEST)
int testdram (void)
{
	uint *pstart = (uint *) CFG_MEMTEST_START;
	uint *pend = (uint *) CFG_MEMTEST_END;
	uint *p;
	ulong *ptr;
	char *s;

	debug("DDR test phase 1:\n");
	for (p = pstart; p < pend; p++)
		*p = 0xaaaaaaaa;

	for (p = pstart; p < pend; p++) {
		if (*p != 0xaaaaaaaa) {
			printf ("DDR test fails at: %08x, val:%08x\n", (uint) p, *p);

			/* reset */
			printf ("reboot\n");

			/* ExtPIO Initialize */
			ptr = (ulong *)EXTPIO_CS5_BASE;
			*(ptr + 1) = 0x00004301;
			*ptr = 0x0001fff0;

			udelay(100);

			s = (char *)EXTPIO_CS5_BASE;
			*(s + 7) = 0x08;
			*(s + 4) = 0x01;
			return 1;
		}
	}

	debug("DDR test phase 2:\n");
	for (p = pstart; p < pend; p++)
		*p = 0x55555555;

	for (p = pstart; p < pend; p++) {
		if (*p != 0x55555555) {
			printf ("DDR test fails at: %08x, val:%08x\n", (uint) p, *p);

			/* reset */
			printf ("reboot\n");

			/* ExtPIO Initialize */
			ptr = (ulong *)EXTPIO_CS5_BASE;
			*(ptr + 1) = 0x00004301;
			*ptr = 0x0001fff0;

			udelay(100);

			s = (char *)EXTPIO_CS5_BASE;
			*(s + 7) = 0x08;
			*(s + 4) = 0x01;
			return 1;
		}
	}

	printf("DDR test passed.\n");
	return 0;
}
#endif

void show_boot_progress(int val)
{
	uchar *ptr;

	ptr = (uchar *)EXTPIO_CS5_BASE + 1;
	*ptr = ((*ptr) & 0xf0) | (char)(val & 0x0f);
	return;
}

/*
 * Initialize Local Bus
 */

void
local_bus_init(void)
{
	volatile ccsr_lbc_t *lbc = (void *)(CONFIG_SYS_MPC85xx_LBC_ADDR);

	uint reg_lcrr;

	/* CLKDIV <- 4 (CCB/8)*/
	reg_lcrr = lbc->lcrr;
	reg_lcrr &= (~0x0000001f);
	reg_lcrr |= (0x00000004);

	lbc->lcrr = reg_lcrr;
	udelay(200);

	asm("sync;isync;msync");
}

#define PES16T4G2_PCIELCTL2_ADDR 0x70
#define PES16T4G2_PCIELCTL2_VAL  0x01
#define PES16T4G2_SWCTL_ADDR 0x404
#define PES16T4G2_SWCTL_BIT  0x00000008
#define PES16T4G2_IOEXPADDR1_ADDR 0x438
#define PES16T4G2_IOEXPADDR1_VAL  0x48
#define PES16T4G2_PHYLSTATE0_ADDR 0x540
#define PES16T4G2_PHYLSTATE0_VAL  0x80000000

void
idt89hpes16t4ag2_init(void)
{
	uint	val4;
	ulong addr = 0, value = 0;
	pci_dev_t bdf = 0;

	debug("89HPES16T4AG2 Initializing\n");

	/* 01.00 */
	bdf = (1 << 16 | 0 << 11 | 0 << 8);

	/* REG UNLOCK */
	addr = PES16T4G2_SWCTL_ADDR;
	pci_read_config_dword(bdf, addr, &val4);
	value = val4| PES16T4G2_SWCTL_BIT;
	pci_write_config_dword(bdf, addr, value);

	/* IO Expander 4 Address */
	addr = PES16T4G2_IOEXPADDR1_ADDR;
	value = PES16T4G2_IOEXPADDR1_VAL;
	pci_write_config_dword(bdf, addr, value);

	/* REG LOCK */
	addr = PES16T4G2_SWCTL_ADDR;
	pci_read_config_dword(bdf, addr, &val4);
	value = val4 & (~PES16T4G2_SWCTL_BIT);
	pci_write_config_dword(bdf, addr, value);


	/* 02.02 */
	bdf = (2 << 16 | 2 << 11 | 0 << 8);

	/* 2.5G(Gen1) */
	addr = PES16T4G2_PCIELCTL2_ADDR;
	value = PES16T4G2_PCIELCTL2_VAL;
	pci_write_config_dword(bdf, addr, value);

	addr = PES16T4G2_PHYLSTATE0_ADDR;
	value = PES16T4G2_PHYLSTATE0_VAL;
	pci_write_config_dword(bdf, addr, value);

	/* 02.03 */
	bdf = (2 << 16 | 3 << 11 | 0 << 8);

	/* 2.5G(Gen1) */
	addr = PES16T4G2_PCIELCTL2_ADDR;
	value = PES16T4G2_PCIELCTL2_VAL;
	pci_write_config_dword(bdf, addr, value);

	addr = PES16T4G2_PHYLSTATE0_ADDR;
	value = PES16T4G2_PHYLSTATE0_VAL;
	pci_write_config_dword(bdf, addr, value);
}

void pu_led_init (void)
{
	char *ptr;
	ptr = (char *)EXTPIO_CS5_BASE;
	*(ptr + 1) = 0x01;
}

void pu_led_toggle (void)
{
	char *ptr;
	ptr = (char *)EXTPIO_CS5_BASE;
	*(ptr + 1) ^= 0x01;
}

void pu_led_set (char mask, int state)
{
	char *ptr;
	ptr = (char *)EXTPIO_CS5_BASE;
	if (state == PU_LED_STATUS_ON)
		*(ptr + 1) |= mask;
	else
		*(ptr + 1) &= ~mask;
}

struct pci_controller *pu_get_pcie1_hose (void)
{
	return &pcie1_hose;
}
