/*
 * (C) Copyright 2000-2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 *  Command Processor Table
 */

#include <common.h>
#include <command.h>
#include <tsec.h>
#include <malloc.h>
#include <fat.h>

int
do_version (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	extern char version_string[];
	printf ("\n%s\n", version_string);
	return 0;
}

U_BOOT_CMD(
	version,	1,		1,	do_version,
	"print monitor version",
	NULL
);

#if defined(CONFIG_CMD_ECHO)

int
do_echo (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int i, putnl = 1;

	for (i = 1; i < argc; i++) {
		char *p = argv[i], c;

		if (i > 1)
			putc(' ');
		while ((c = *p++) != '\0') {
			if (c == '\\' && *p == 'c') {
				putnl = 0;
				p++;
			} else {
				putc(c);
			}
		}
	}

	if (putnl)
		putc('\n');
	return 0;
}

U_BOOT_CMD(
	echo,	CONFIG_SYS_MAXARGS,	1,	do_echo,
	"echo args to console",
	"[args..]\n"
	"    - echo args to console; \\c suppresses newline\n"
);

#endif

#ifdef CONFIG_SYS_HUSH_PARSER

int
do_test (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char **ap;
	int left, adv, expr, last_expr, neg, last_cmp;

	/* args? */
	if (argc < 3)
		return 1;

#if 0
	{
		printf("test:");
		left = 1;
		while (argv[left])
			printf(" %s", argv[left++]);
	}
#endif

	last_expr = 0;
	left = argc - 1; ap = argv + 1;
	if (left > 0 && strcmp(ap[0], "!") == 0) {
		neg = 1;
		ap++;
		left--;
	} else
		neg = 0;

	expr = -1;
	last_cmp = -1;
	last_expr = -1;
	while (left > 0) {

		if (strcmp(ap[0], "-o") == 0 || strcmp(ap[0], "-a") == 0)
			adv = 1;
		else if (strcmp(ap[0], "-z") == 0 || strcmp(ap[0], "-n") == 0)
			adv = 2;
		else
			adv = 3;

		if (left < adv) {
			expr = 1;
			break;
		}

		if (adv == 1) {
			if (strcmp(ap[0], "-o") == 0) {
				last_expr = expr;
				last_cmp = 0;
			} else if (strcmp(ap[0], "-a") == 0) {
				last_expr = expr;
				last_cmp = 1;
			} else {
				expr = 1;
				break;
			}
		}

		if (adv == 2) {
			if (strcmp(ap[0], "-z") == 0)
				expr = strlen(ap[1]) == 0 ? 1 : 0;
			else if (strcmp(ap[0], "-n") == 0)
				expr = strlen(ap[1]) == 0 ? 0 : 1;
			else {
				expr = 1;
				break;
			}

			if (last_cmp == 0)
				expr = last_expr || expr;
			else if (last_cmp == 1)
				expr = last_expr && expr;
			last_cmp = -1;
		}

		if (adv == 3) {
			if (strcmp(ap[1], "=") == 0)
				expr = strcmp(ap[0], ap[2]) == 0;
			else if (strcmp(ap[1], "!=") == 0)
				expr = strcmp(ap[0], ap[2]) != 0;
			else if (strcmp(ap[1], ">") == 0)
				expr = strcmp(ap[0], ap[2]) > 0;
			else if (strcmp(ap[1], "<") == 0)
				expr = strcmp(ap[0], ap[2]) < 0;
			else if (strcmp(ap[1], "-eq") == 0)
				expr = simple_strtol(ap[0], NULL, 10) == simple_strtol(ap[2], NULL, 10);
			else if (strcmp(ap[1], "-ne") == 0)
				expr = simple_strtol(ap[0], NULL, 10) != simple_strtol(ap[2], NULL, 10);
			else if (strcmp(ap[1], "-lt") == 0)
				expr = simple_strtol(ap[0], NULL, 10) < simple_strtol(ap[2], NULL, 10);
			else if (strcmp(ap[1], "-le") == 0)
				expr = simple_strtol(ap[0], NULL, 10) <= simple_strtol(ap[2], NULL, 10);
			else if (strcmp(ap[1], "-gt") == 0)
				expr = simple_strtol(ap[0], NULL, 10) > simple_strtol(ap[2], NULL, 10);
			else if (strcmp(ap[1], "-ge") == 0)
				expr = simple_strtol(ap[0], NULL, 10) >= simple_strtol(ap[2], NULL, 10);
			else {
				expr = 1;
				break;
			}

			if (last_cmp == 0)
				expr = last_expr || expr;
			else if (last_cmp == 1)
				expr = last_expr && expr;
			last_cmp = -1;
		}

		ap += adv; left -= adv;
	}

	if (neg)
		expr = !expr;

	expr = !expr;

	debug (": returns %d\n", expr);

	return expr;
}

U_BOOT_CMD(
	test,	CONFIG_SYS_MAXARGS,	1,	do_test,
	"minimal test like /bin/sh",
	"[args..]\n"
	"    - test functionality\n"
);

int
do_exit (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int r;

	r = 0;
	if (argc > 1)
		r = simple_strtoul(argv[1], NULL, 10);

	return -r - 2;
}

U_BOOT_CMD(
	exit,	2,	1,	do_exit,
	"exit script",
	"    - exit functionality\n"
);


#endif

/*
 * Use puts() instead of printf() to avoid printf buffer overflow
 * for long help messages
 */
int do_help (cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	int i;
	int rcode = 0;

	if (argc == 1) {	/*show list of commands */

		int cmd_items = &__u_boot_cmd_end -
				&__u_boot_cmd_start;	/* pointer arith! */
		cmd_tbl_t *cmd_array[cmd_items];
		int i, j, swaps;

		/* Make array of commands from .uboot_cmd section */
		cmdtp = &__u_boot_cmd_start;
		for (i = 0; i < cmd_items; i++) {
			cmd_array[i] = cmdtp++;
		}

		/* Sort command list (trivial bubble sort) */
		for (i = cmd_items - 1; i > 0; --i) {
			swaps = 0;
			for (j = 0; j < i; ++j) {
				if (strcmp (cmd_array[j]->name,
					    cmd_array[j + 1]->name) > 0) {
					cmd_tbl_t *tmp;
					tmp = cmd_array[j];
					cmd_array[j] = cmd_array[j + 1];
					cmd_array[j + 1] = tmp;
					++swaps;
				}
			}
			if (!swaps)
				break;
		}

		/* print short help (usage) */
		for (i = 0; i < cmd_items; i++) {
			const char *usage = cmd_array[i]->usage;

			/* allow user abort */
			if (ctrlc ())
				return 1;
			if (usage == NULL)
				continue;
			printf("%-*s- %s\n", CONFIG_SYS_HELP_CMD_WIDTH,
			       cmd_array[i]->name, usage);
		}
		return 0;
	}
	/*
	 * command help (long version)
	 */
	for (i = 1; i < argc; ++i) {
		if ((cmdtp = find_cmd (argv[i])) != NULL) {
#ifdef	CONFIG_SYS_LONGHELP
			/* found - print (long) help info */
			puts (cmdtp->name);
			putc (' ');
			if (cmdtp->help) {
				puts (cmdtp->help);
			} else {
				puts ("- No help available.\n");
				rcode = 1;
			}
			putc ('\n');
#else	/* no long help available */
			if (cmdtp->usage)
				printf ("%s - %s\n", cmdtp->name, cmdtp->usage);
#endif	/* CONFIG_SYS_LONGHELP */
		} else {
			printf ("Unknown command '%s' - try 'help'"
				" without arguments for list of all"
				" known commands\n\n", argv[i]
					);
			rcode = 1;
		}
	}
	return rcode;
}


U_BOOT_CMD(
	help,	CONFIG_SYS_MAXARGS,	1,	do_help,
	"print online help",
	"[command ...]\n"
	"    - show help information (for 'command')\n"
	"'help' prints online help for the monitor commands.\n\n"
	"Without arguments, it prints a short usage message for all commands.\n\n"
	"To get detailed help information for specific commands you can type\n"
  "'help' with one or more command names as arguments.\n"
);

/* This do not ust the U_BOOT_CMD macro as ? can't be used in symbol names */
#ifdef  CONFIG_SYS_LONGHELP
cmd_tbl_t __u_boot_cmd_question_mark Struct_Section = {
	"?",	CONFIG_SYS_MAXARGS,	1,	do_help,
	"alias for 'help'",
	NULL
};
#else
cmd_tbl_t __u_boot_cmd_question_mark Struct_Section = {
	"?",	CONFIG_SYS_MAXARGS,	1,	do_help,
	"alias for 'help'"
};
#endif /* CONFIG_SYS_LONGHELP */

/***************************************************************************
 * find command table entry for a command
 */
cmd_tbl_t *find_cmd_tbl (const char *cmd, cmd_tbl_t *table, int table_len)
{
	cmd_tbl_t *cmdtp;
	cmd_tbl_t *cmdtp_temp = table;	/*Init value */
	const char *p;
	int len;
	int n_found = 0;

	/*
	 * Some commands allow length modifiers (like "cp.b");
	 * compare command name only until first dot.
	 */
	len = ((p = strchr(cmd, '.')) == NULL) ? strlen (cmd) : (p - cmd);

	for (cmdtp = table;
	     cmdtp != table + table_len;
	     cmdtp++) {
		if (strncmp (cmd, cmdtp->name, len) == 0) {
			if (len == strlen (cmdtp->name))
				return cmdtp;	/* full match */

			cmdtp_temp = cmdtp;	/* abbreviated command ? */
			n_found++;
		}
	}
	if (n_found == 1) {			/* exactly one match */
		return cmdtp_temp;
	}

	return NULL;	/* not found or ambiguous command */
}

cmd_tbl_t *find_cmd (const char *cmd)
{
	int len = &__u_boot_cmd_end - &__u_boot_cmd_start;
	return find_cmd_tbl(cmd, &__u_boot_cmd_start, len);
}

void cmd_usage(cmd_tbl_t *cmdtp)
{
	printf("Usage:\n%s - %s\n\n", cmdtp->name, cmdtp->usage);
}

#ifdef CONFIG_AUTO_COMPLETE

int var_complete(int argc, char *argv[], char last_char, int maxv, char *cmdv[])
{
	static char tmp_buf[512];
	int space;

	space = last_char == '\0' || last_char == ' ' || last_char == '\t';

	if (space && argc == 1)
		return env_complete("", maxv, cmdv, sizeof(tmp_buf), tmp_buf);

	if (!space && argc == 2)
		return env_complete(argv[1], maxv, cmdv, sizeof(tmp_buf), tmp_buf);

	return 0;
}

static void install_auto_complete_handler(const char *cmd,
		int (*complete)(int argc, char *argv[], char last_char, int maxv, char *cmdv[]))
{
	cmd_tbl_t *cmdtp;

	cmdtp = find_cmd(cmd);
	if (cmdtp == NULL)
		return;

	cmdtp->complete = complete;
}

void install_auto_complete(void)
{
	install_auto_complete_handler("printenv", var_complete);
	install_auto_complete_handler("setenv", var_complete);
#if defined(CONFIG_CMD_RUN)
	install_auto_complete_handler("run", var_complete);
#endif
}

/*************************************************************************************/

static int complete_cmdv(int argc, char *argv[], char last_char, int maxv, char *cmdv[])
{
	cmd_tbl_t *cmdtp;
	const char *p;
	int len, clen;
	int n_found = 0;
	const char *cmd;

	/* sanity? */
	if (maxv < 2)
		return -2;

	cmdv[0] = NULL;

	if (argc == 0) {
		/* output full list of commands */
		for (cmdtp = &__u_boot_cmd_start; cmdtp != &__u_boot_cmd_end; cmdtp++) {
			if (n_found >= maxv - 2) {
				cmdv[n_found++] = "...";
				break;
			}
			cmdv[n_found++] = cmdtp->name;
		}
		cmdv[n_found] = NULL;
		return n_found;
	}

	/* more than one arg or one but the start of the next */
	if (argc > 1 || (last_char == '\0' || last_char == ' ' || last_char == '\t')) {
		cmdtp = find_cmd(argv[0]);
		if (cmdtp == NULL || cmdtp->complete == NULL) {
			cmdv[0] = NULL;
			return 0;
		}
		return (*cmdtp->complete)(argc, argv, last_char, maxv, cmdv);
	}

	cmd = argv[0];
	/*
	 * Some commands allow length modifiers (like "cp.b");
	 * compare command name only until first dot.
	 */
	p = strchr(cmd, '.');
	if (p == NULL)
		len = strlen(cmd);
	else
		len = p - cmd;

	/* return the partial matches */
	for (cmdtp = &__u_boot_cmd_start; cmdtp != &__u_boot_cmd_end; cmdtp++) {

		clen = strlen(cmdtp->name);
		if (clen < len)
			continue;

		if (memcmp(cmd, cmdtp->name, len) != 0)
			continue;

		/* too many! */
		if (n_found >= maxv - 2) {
			cmdv[n_found++] = "...";
			break;
		}

		cmdv[n_found++] = cmdtp->name;
	}

	cmdv[n_found] = NULL;
	return n_found;
}

static int make_argv(char *s, int argvsz, char *argv[])
{
	int argc = 0;

	/* split into argv */
	while (argc < argvsz - 1) {

		/* skip any white space */
		while ((*s == ' ') || (*s == '\t'))
			++s;

		if (*s == '\0')	/* end of s, no more args	*/
			break;

		argv[argc++] = s;	/* begin of argument string	*/

		/* find end of string */
		while (*s && (*s != ' ') && (*s != '\t'))
			++s;

		if (*s == '\0')		/* end of s, no more args	*/
			break;

		*s++ = '\0';		/* terminate current arg	 */
	}
	argv[argc] = NULL;

	return argc;
}

static void print_argv(const char *banner, const char *leader, const char *sep, int linemax, char *argv[])
{
	int ll = leader != NULL ? strlen(leader) : 0;
	int sl = sep != NULL ? strlen(sep) : 0;
	int len, i;

	if (banner) {
		puts("\n");
		puts(banner);
	}

	i = linemax;	/* force leader and newline */
	while (*argv != NULL) {
		len = strlen(*argv) + sl;
		if (i + len >= linemax) {
			puts("\n");
			if (leader)
				puts(leader);
			i = ll - sl;
		} else if (sep)
			puts(sep);
		puts(*argv++);
		i += len;
	}
	printf("\n");
}

static int find_common_prefix(char *argv[])
{
	int i, len;
	char *anchor, *s, *t;

	if (*argv == NULL)
		return 0;

	/* begin with max */
	anchor = *argv++;
	len = strlen(anchor);
	while ((t = *argv++) != NULL) {
		s = anchor;
		for (i = 0; i < len; i++, t++, s++) {
			if (*t != *s)
				break;
		}
		len = s - anchor;
	}
	return len;
}

static char tmp_buf[CONFIG_SYS_CBSIZE];	/* copy of console I/O buffer	*/

int cmd_auto_complete(const char *const prompt, char *buf, int *np, int *colp)
{
	int n = *np, col = *colp;
	char *argv[CONFIG_SYS_MAXARGS + 1];		/* NULL terminated	*/
	char *cmdv[20];
	char *s, *t;
	const char *sep;
	int i, j, k, len, seplen, argc;
	int cnt;
	char last_char;

	if (strcmp(prompt, CONFIG_SYS_PROMPT) != 0)
		return 0;	/* not in normal console */

	cnt = strlen(buf);
	if (cnt >= 1)
		last_char = buf[cnt - 1];
	else
		last_char = '\0';

	/* copy to secondary buffer which will be affected */
	strcpy(tmp_buf, buf);

	/* separate into argv */
	argc = make_argv(tmp_buf, sizeof(argv)/sizeof(argv[0]), argv);

	/* do the completion and return the possible completions */
	i = complete_cmdv(argc, argv, last_char, sizeof(cmdv)/sizeof(cmdv[0]), cmdv);

	/* no match; bell and out */
	if (i == 0) {
		if (argc > 1)	/* allow tab for non command */
			return 0;
		putc('\a');
		return 1;
	}

	s = NULL;
	len = 0;
	sep = NULL;
	seplen = 0;
	if (i == 1) { /* one match; perfect */
		k = strlen(argv[argc - 1]);
		s = cmdv[0] + k;
		len = strlen(s);
		sep = " ";
		seplen = 1;
	} else if (i > 1 && (j = find_common_prefix(cmdv)) != 0) {	/* more */
		k = strlen(argv[argc - 1]);
		j -= k;
		if (j > 0) {
			s = cmdv[0] + k;
			len = j;
		}
	}

	if (s != NULL) {
		k = len + seplen;
		/* make sure it fits */
		if (n + k >= CONFIG_SYS_CBSIZE - 2) {
			putc('\a');
			return 1;
		}

		t = buf + cnt;
		for (i = 0; i < len; i++)
			*t++ = *s++;
		if (sep != NULL)
			for (i = 0; i < seplen; i++)
				*t++ = sep[i];
		*t = '\0';
		n += k;
		col += k;
		puts(t - k);
		if (sep == NULL)
			putc('\a');
		*np = n;
		*colp = col;
	} else {
		print_argv(NULL, "  ", " ", 78, cmdv);

		puts(prompt);
		puts(buf);
	}
	return 1;
}

#endif

#ifdef CMD_DATA_SIZE
int cmd_get_data_size(char* arg, int default_size)
{
	/* Check for a size specification .b, .w or .l.
	 */
	int len = strlen(arg);
	if (len > 2 && arg[len-2] == '.') {
		switch(arg[len-1]) {
		case 'b':
			return 1;
		case 'w':
			return 2;
		case 'l':
			return 4;
		case 's':
			return -2;
		default:
			return -1;
		}
	}
	return default_size;
}
#endif

#define write_phy_reg(priv, regnum, value) tsec_local_mdio_write(priv->phyregs,priv->phyaddr,regnum,value)
// extern void write_phy_reg(struct tsec_private *priv, uint regnum, uint value);
extern void tsec_local_mdio_write(volatile tsec_t *phyregs, uint addr,	uint reg, uint value);

extern uint read_phy_reg(struct tsec_private *priv, uint regnum);

int
do_physet (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong	port, addr, writeval;
	struct tsec_private *priv;
	int	size;

	if (argc != 4){
		printf ("my %d Usage:\n%s\n", argc, cmdtp->usage);
		return 1;
	}

	/* Check for size specification.
	*/
	if ((size = cmd_get_data_size(argv[0], 4)) < 1)
		return 1;

	/* select eth0 or eth1 */
	port = simple_strtoul(argv[1], NULL, 16);

	/* Address is specified since argc > 1
	*/
	addr = simple_strtoul(argv[2], NULL, 16);

	/* Get the value to write.
	*/
	writeval = simple_strtoul(argv[3], NULL, 16);

	priv = (struct tsec_private *) malloc(sizeof(*priv));
	priv->regs = (volatile tsec_t *)(TSEC_BASE_ADDR);
	priv->phyregs = (volatile tsec_t *)(TSEC_BASE_ADDR +
										TSEC1_PHYIDX*TSEC_SIZE);

	if(port == 1){
		priv->phyaddr = TSEC2_PHY_ADDR;
	}else{
		priv->phyaddr = TSEC1_PHY_ADDR;
	}
	priv->flags = 1;

	write_phy_reg(priv, addr, writeval);

	return 0;
}

U_BOOT_CMD(
	physet,     4,     1,      do_physet,
	"phy port reg modify",
	"address\n" " - memory modify\n"
);

extern int Fpga_Update(long);

int
do_usbflash (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	extern int do_usb(cmd_tbl_t *, int, int, char *[]);
	extern int do_fat_ls(cmd_tbl_t *, int, int, char *[]);
	extern int do_fat_fsload(cmd_tbl_t *, int, int, char *[]);
	extern int do_flerase(cmd_tbl_t *, int, int, char *[]);
	extern int do_mem_cp(cmd_tbl_t *, int, int, char *[]);
	extern int do_protect (cmd_tbl_t *, int, int, char *[]);
	extern int enable_pupld2_vup(void);
	extern int disable_pupld2_vup(void);
	extern int enable_cnpld_vup(void);
	extern int disable_cnpld_vup(void);
	extern int FPLD_FlashUpdate(char, unsigned long, unsigned long);
	char *local_args[3], *fatload_args[6], *erase_args[4], *cp_args[5], *fatls_args[5], *protect_args[4];
	int verify_flash(uint, uint, uint);
	int mymot2bin(int, int, char *);

	int rcode;
	char *s, buf[100], *ptr;
	int mysize;

#ifdef BOARD_CPU453
	char r1000_dir[] = "/SONY/PRO/SRMASTER/R1000/";
#else
	char srpc_dir[] = "/SONY/PRO/SRMASTER/SRPC5/";

	/* Front LED ExtPIO Initialize */
	ptr = (char *)(PU_PLD3_PWRLED_ADDR);
	*ptr = 0x40;
#endif

	disable_interrupts();

	local_args[0] = "usb";
	local_args[1] = "start";
	local_args[2] = NULL;
	rcode = do_usb(cmdtp, 0, 2, local_args);

	if(rcode){

#ifdef BOARD_CPU448
		ptr = (char *)(PU_PLD3_PWRLED_ADDR);
		*ptr = 0x00;
#endif

		enable_interrupts();
		return 1;
	}

#ifdef PU_LOAD_IMAGE_FILE

	fatload_args[0] = "fatload";
	fatload_args[1] = "usb";
	fatload_args[2] = "0:1";
	fatload_args[3] = "1000000";
	fatload_args[4] = "/SONY/PRO/SRMASTER/SRPC5/u-boot.bin";
	fatload_args[5] = NULL;
	rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);

	if(rcode == 0){
		protect_args[0] = "protect";
		protect_args[1] = "off";
		protect_args[2] = "all";
		protect_args[3] = NULL;
		do_protect(cmdtp, 0, 3, protect_args);

		erase_args[0] = "erase";
		erase_args[1] = "eff60000";
		erase_args[2] = "efffffff";
		erase_args[3] = NULL;
		do_flerase(cmdtp, 0, 3, erase_args);

		cp_args[0] = "cp.b";
		cp_args[1] = "1000000";
		cp_args[2] = "eff80000";
		cp_args[3] = getenv("filesize");
		cp_args[4] = NULL;
		do_mem_cp(cmdtp, 0, 4, cp_args);

		verify_flash(0x1000000, 0xeff80000, getenv("filesize"));
	}

	fatload_args[0] = "fatload";
	fatload_args[1] = "usb";
	fatload_args[2] = "0:1";
	fatload_args[3] = "1000000";
	fatload_args[4] = "uimage";
	fatload_args[5] = NULL;
	debug("mycmd = %s %s %s %s %s\n", fatload_args[0], fatload_args[1], fatload_args[2], fatload_args[3], fatload_args[4]);
	rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);

	s = getenv("filesize");
	mysize = (int)simple_strtol(s, NULL, 16);
	debug("rcode: %d, fenv: %s, val: %d\n", rcode, s, mysize);

	if(rcode == 0){
		erase_args[0] = "erase";
		erase_args[1] = "ef040000";
		erase_args[2] = "ef3fffff";
		erase_args[3] = NULL;
		debug("mycmd = %s %s %s\n", erase_args[0], erase_args[1], erase_args[2]);
		do_flerase(cmdtp, 0, 3, erase_args);

		cp_args[0] = "cp.b";
		cp_args[1] = "1000000";
		cp_args[2] = "ef040000";
		cp_args[3] = getenv("filesize");
		cp_args[4] = NULL;
		debug("mycmd = %s %s %s %s\n", cp_args[0], cp_args[1], cp_args[2], cp_args[3]);
		do_mem_cp(cmdtp, 0, 4, cp_args);
	}

	fatload_args[4] = "rootfs.ext2.gz.uboot";
	debug("mycmd = %s %s %s %s %s\n", fatload_args[0], fatload_args[1], fatload_args[2], fatload_args[3], fatload_args[4]);
	rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);

	s = getenv("filesize");
	mysize = (int)simple_strtol(s, NULL, 16);
	debug("rcode: %d, fenv: %s, val: %d\n", rcode, s, mysize);

	if(rcode == 0){
		erase_args[0] = "erase";
		erase_args[1] = "ef400000";
		erase_args[2] = "efbfffff";
		erase_args[3] = NULL;
		debug("mycmd = %s %s %s\n", erase_args[0], erase_args[1], erase_args[2]);
		do_flerase(cmdtp, 0, 3, erase_args);

		cp_args[0] = "cp.b";
		cp_args[1] = "1000000";
		cp_args[2] = "ef400000";
		cp_args[3] = getenv("filesize");
		cp_args[4] = NULL;
		debug("mycmd = %s %s %s %s\n", cp_args[0], cp_args[1], cp_args[2], cp_args[3]);
		do_mem_cp(cmdtp, 0, 4, cp_args);
	}

	fatload_args[4] = "p2020rdb.dtb";
	debug("mycmd = %s %s %s %s %s\n", fatload_args[0], fatload_args[1], fatload_args[2], fatload_args[3], fatload_args[4]);
	rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);

	s = getenv("filesize");
	mysize = (int)simple_strtol(s, NULL, 16);
	debug("rcode: %d, fenv: %s, val: %d\n", rcode, s, mysize);

	if(rcode == 0){
		erase_args[0] = "erase";
		erase_args[1] = "ef000000";
		erase_args[2] = "ef03ffff";
		erase_args[3] = NULL;
		debug("mycmd = %s %s %s\n", erase_args[0], erase_args[1], erase_args[2]);
		do_flerase(cmdtp, 0, 3, erase_args);

		cp_args[0] = "cp.b";
		cp_args[1] = "1000000";
		cp_args[2] = "ef000000";
		cp_args[3] = getenv("filesize");
		cp_args[4] = NULL;
		debug("mycmd = %s %s %s %s\n", cp_args[0], cp_args[1], cp_args[2], cp_args[3]);
		do_mem_cp(cmdtp, 0, 4, cp_args);
	}
#endif

	fatls_args[0] = "fatls";
	fatls_args[1] = "usb";
	fatls_args[2] = "0:1";

#ifdef BOARD_CPU453
	fatls_args[3] = r1000_dir;
#else
	fatls_args[3] = srpc_dir;
#endif

	fatls_args[4] = NULL;
	rcode = do_fat_ls(cmdtp, 0, 4, fatls_args);

	if((argc == 1) || (strcmp(argv[1], "boot") == 0)){
		if(argc == 3) setenv("bootmotfile", argv[2]);
		if((s = getenv("bootmotfile")) != NULL){
			sprintf(buf, "%s%s",fatls_args[3], s);
			fatload_args[0] = "fatload";
			fatload_args[1] = "usb";
			fatload_args[2] = "0:1";
			fatload_args[3] = "1000000";
			fatload_args[4] = buf;
			fatload_args[5] = NULL;
			rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);
			
			s = getenv("filesize");
			mysize = (int)simple_strtol(s, NULL, 16);
			
			if(rcode == 0){
				mysize = mymot2bin(0x1000000, 0x10000000, "BOOT");
				if(mysize){
					protect_args[0] = "protect";
					protect_args[1] = "off";
					protect_args[2] = "all";
					protect_args[3] = NULL;
					do_protect(cmdtp, 0, 3, protect_args);
					
					erase_args[0] = "erase";
					erase_args[1] = "eff60000";
					erase_args[2] = "efffffff";
					erase_args[3] = NULL;
					do_flerase(cmdtp, 0, 3, erase_args);
					
					sprintf(buf, "%x", mysize);
					cp_args[0] = "cp.b";
					cp_args[1] = "10000000";
					cp_args[2] = "eff80000";
					cp_args[3] = buf;
					cp_args[4] = NULL;
					do_mem_cp(cmdtp, 0, 4, cp_args);

					verify_flash(0x10000000, 0xeff80000, mysize);
				}
			}
		}
	}

	if((argc == 1) || (strcmp(argv[1], "kernel") == 0)){
		if(argc == 3) setenv("kernelmotfile", argv[2]);
		if((s = getenv("kernelmotfile")) != NULL){
			sprintf(buf, "%s%s",fatls_args[3], s);
			fatload_args[0] = "fatload";
			fatload_args[1] = "usb";
			fatload_args[2] = "0:1";
			fatload_args[3] = "1000000";
			fatload_args[4] = buf;
			fatload_args[5] = NULL;
			rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);
			
			s = getenv("filesize");
			mysize = (int)simple_strtol(s, NULL, 16);
			
			if(rcode == 0){
				mysize = mymot2bin(0x1000000, 0x10000000, "KERNEL");
				if(mysize > 0x400000){
					printf("mot-file is too large\n");
					mysize = 0;
				}
				if(mysize){
					erase_args[0] = "erase";
					erase_args[1] = "ef000000";
					erase_args[2] = "ef3fffff";
					erase_args[3] = NULL;
					do_flerase(cmdtp, 0, 3, erase_args);
					
					sprintf(buf, "%x", mysize);
					cp_args[0] = "cp.b";
					cp_args[1] = "10000000";
					cp_args[2] = "ef000000";
					cp_args[3] = buf;
					cp_args[4] = NULL;
					do_mem_cp(cmdtp, 0, 4, cp_args);

					verify_flash(0x10000000, 0xef000000, mysize);
				}
			}
		}
	}
	
	if((argc == 1) || (strcmp(argv[1], "ramdisk") == 0)){
		if(argc == 3) setenv("ramdiskmotfile", argv[2]);
		if((s = getenv("ramdiskmotfile")) != NULL){
			sprintf(buf, "%s%s",fatls_args[3], s);
			fatload_args[0] = "fatload";
			fatload_args[1] = "usb";
			fatload_args[2] = "0:1";
			fatload_args[3] = "1000000";
			fatload_args[4] = buf;
			fatload_args[5] = NULL;
			rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);
			
			s = getenv("filesize");
			mysize = (int)simple_strtol(s, NULL, 16);
			
			if(rcode == 0){
				mysize = mymot2bin(0x1000000, 0x10000000, "RAMDISK");
				if(mysize > 0xb00000){
					printf("mot-file is too large\n");
					mysize = 0;
				}
				if(mysize){
					erase_args[0] = "erase";
					erase_args[1] = "ef400000";
					erase_args[2] = "efefffff";
					erase_args[3] = NULL;
					do_flerase(cmdtp, 0, 3, erase_args);
					
					sprintf(buf, "%x", mysize);
					cp_args[0] = "cp.b";
					cp_args[1] = "10000000";
					cp_args[2] = "ef400000";
					cp_args[3] = buf;
					cp_args[4] = NULL;
					do_mem_cp(cmdtp, 0, 4, cp_args);

					verify_flash(0x10000000, 0xef400000, mysize);
				}
			}
		}
	}

	if((argc == 1) || (strcmp(argv[1], "pld3") == 0)){
		if(argc == 3) setenv("pld3motfile", argv[2]);
		if((s = getenv("pld3motfile")) != NULL){
			sprintf(buf, "%s%s",fatls_args[3], s);
			fatload_args[0] = "fatload";
			fatload_args[1] = "usb";
			fatload_args[2] = "0:1";
			fatload_args[3] = "1000000";
			fatload_args[4] = buf;
			fatload_args[5] = NULL;
			rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);
			
			if(rcode == 0){
				s = getenv("filesize");
				mysize = (int)simple_strtol(s, NULL, 16);
				ptr = (char *)(mysize + 0x1000000);
				*ptr = 0x00;
				mysize = mymot2bin(0x1000000, 0x10000000, "PLD3");
				
				enable_cnpld_vup();
				Fpga_Update(mysize);
				disable_cnpld_vup();
			}
		}
	}

	if((argc == 1) || (strcmp(argv[1], "pld2") == 0)){
		if(argc == 3) setenv("pld2motfile", argv[2]);
		if((s = getenv("pld2motfile")) != NULL){
			sprintf(buf, "%s%s",fatls_args[3], s);
			fatload_args[0] = "fatload";
			fatload_args[1] = "usb";
			fatload_args[2] = "0:1";
			fatload_args[3] = "1000000";
			fatload_args[4] = buf;
			fatload_args[5] = NULL;
			rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);
			
			if(rcode == 0){
				s = getenv("filesize");
				mysize = (int)simple_strtol(s, NULL, 16);
				ptr = (char *)(mysize + 0x1000000);
				*ptr = 0x00;
				mysize = mymot2bin(0x1000000, 0x10000000, "PLD2");
				
				enable_pupld2_vup();
				Fpga_Update(mysize);
				disable_pupld2_vup();
			}
		}
	}

	if((argc == 1) || (strcmp(argv[1], "pld1") == 0)){
		if(argc == 3) setenv("pld1motfile", argv[2]);
		if((s = getenv("pld1motfile")) != NULL){
			sprintf(buf, "%s%s",fatls_args[3], s);
			fatload_args[0] = "fatload";
			fatload_args[1] = "usb";
			fatload_args[2] = "0:1";
			fatload_args[3] = "1000000";
			fatload_args[4] = buf;
			fatload_args[5] = NULL;
			rcode = do_fat_fsload(cmdtp, 0, 5, fatload_args);
			
			if(rcode == 0){
				s = getenv("filesize");
				mysize = (int)simple_strtol(s, NULL, 16);
				ptr = (char *)(mysize + 0x1000000);
				*ptr = 0x00;
				
				mysize = mymot2bin(0x1000000, 0x10000000, "PLD1");
				printf("\npld1 write size = %x\n", mysize);
				FPLD_FlashUpdate(0, mysize, 0x1000);
			}
		}
	}

#ifdef BOARD_CPU448
	ptr = (char *)(PU_PLD3_PWRLED_ADDR);
	*ptr = 0x00;
#endif

	enable_interrupts();
	
	return 0;
}

int verify_flash(uint src, uint dst, uint size)
{
	int i;
	uchar *psrc, *pdst;

	psrc = (uchar *)src;
	pdst = (uchar *)dst;

	for(i = 0; i < size; i++){
		if(*psrc++ != *pdst++){
			printf("Verify NG! %08x: %02x %02x\n", (uint)pdst, (uint)*psrc, (uint)*pdst);
			return 1;
		}
	}

	return 0;
}


#define MOT_FIRMREC_TARGET      1
#define MOT_FIRMREC_VERSION     2
#define MOT_FIRMREC_VTR         5
#define MOT_FIRMREC_TYPE        6
#define MOT_FIRMREC_SIZE        7
int mymot2bin(int src, int dst, char *type)
{
	char *myfgets(char *, int, char **);
	
	unsigned long *pfil;
	char buf[1000], cnvbuf[10];
	char *f, *ptr;
	int line = 0;
	unsigned long bytecnt, data, sum;
	unsigned char cs_h;
	unsigned long addrdata;
	int i;
	
	int reccnt = 0;
	char recbuf[100];

	pfil = (unsigned long *)(dst - (dst % 4));
	for(i = 0; i < 0x4000000 / 4; i++) *pfil++ = 0xffffffff;

	f = (char *)src;
	ptr = (char *)dst;

	while(myfgets(buf, 1000, &f) != NULL){
		if(buf[0] != 'S') { 
			printf("Illegal S-format record.line: %d\n", line); 
			return 0;
		} 

		strncpy(cnvbuf, buf+2, 2);
		cnvbuf[2] = '\0'; 
		bytecnt = simple_strtoul(cnvbuf, NULL, 16); 
		if (strlen(buf) != bytecnt * 2 + 4) { 
			printf("Byte-count error.line: %d(%d != %d)\n%s", line, strlen(buf), (int)(bytecnt * 2 + 4), buf);
			return 0;
		} 

		switch(buf[1]){
		case '0':
			for(i = 0; i < 100; i++){
				strncpy(cnvbuf, buf + (i + 4)*2, 2);
				recbuf[i] = (char)simple_strtoul(cnvbuf, NULL, 16); 
				if(recbuf[i] == 0) break;
			}
            switch(reccnt){
            case MOT_FIRMREC_VERSION:
                printf("Version %s\n",recbuf);
                break;
            case MOT_FIRMREC_VTR:
                printf("VTR %s\n",recbuf);
/*				if(strncmp(recbuf, "SRPC-5", 9)){
					printf("VTR error\n");
					return 0;
				}
*/
                break;
            case MOT_FIRMREC_TARGET:
                printf("TARGET %s\n",recbuf);
#ifdef BOARD_CPU453
				if(strncmp(recbuf, "CPU453", 6)){
					printf("TARGET error\n");
					return 0;
				}
#else
				if((strncmp(recbuf, "PU132", 5)) && (strncmp(recbuf, "CPU448", 6))){
					printf("TARGET error\n");
					return 0;
				}
#endif
                break;
            case MOT_FIRMREC_TYPE:
                printf("TYPE %s\n",recbuf);
				if(strcmp(recbuf, type)){
					printf("TYPE error\n");
					return 0;
				}
				break;
			case MOT_FIRMREC_SIZE:
                printf("SIZE %s\n",recbuf);
				break;
			default:
                break;
			}
			reccnt++;
			break;
		case '7':
			printf("size: %d, %x\n",(int)ptr - dst, (int)ptr - dst);
			return (int)ptr - dst;
		case '3':
			cs_h = 0xff;
			strncpy(cnvbuf, buf+4, 8);
			cnvbuf[8] = '\0'; 
			addrdata = simple_strtoul(cnvbuf, NULL, 16); 

			ptr = (char *)(dst + addrdata);
			for(i = 0; i < bytecnt; i++){
				strncpy(cnvbuf, buf+2 + i*2, 2);					
				cnvbuf[2] = '\0'; 
				data = simple_strtoul(cnvbuf, NULL, 16); 
				if(i >= 5) *ptr++ = data;
				cs_h -= data; 
			}
			strncpy(cnvbuf, buf+2 + bytecnt*2, 2);
			cnvbuf[2] = '\0'; 
			sum = simple_strtoul(cnvbuf, NULL, 16);
			if(cs_h != sum){
				printf("Check Sum error.line: %d, %02x:%02x\n", line, (int)cs_h, (int)sum);
				return 0;
			}
			break;
		default:
			printf("Illegal record.line: %d\n", line); 
			return 0;
		}
		line++;
	}
	printf("size: %d, %x\n",(int)ptr - dst, (int)ptr - dst);
	return (int)ptr - dst;
}

char *myfgets(char *s, int n, char **f)
{
	char c = 0;
	char *cs;

	cs = s;
	while(--n > 0 && (c = **f) != '\0'){
		*f = *f + 1;
		if(c == '\n') break;
		*cs++ = c;
	}
	if(c == '\0' && cs == s) return NULL;
	*cs++ = '\0';
	return s;
}

U_BOOT_CMD(
	usbflash,     3,     1,      do_usbflash,
	"write usb to flash rom",
	"\n"
);


int do_dumpdpram(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong	addr, size, count;
	int i;
	char *s;

	if (argc != 3){
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	addr = simple_strtoul(argv[1], NULL, 16);
	size = simple_strtoul(argv[2], NULL, 16);

	if(addr >= 0x2000){
		printf ("Address Error\n");
		return 1;
	}
	if(addr + size >= 0x2000){
		printf ("Size Error\n");
		return 1;
	}

	s = (char *)0xf90002d8;
	*s = (char)addr;
	*(s + 1) = (char)(addr >> 8);

	s = (char *)0xf90002db;


	count = 0;
	for(i = 0; i < size; i++){
		udelay(1);
		if(count == 0) printf("%04x :", (uint)addr);
		printf("%02x ", *s);
		if(++count >= 16){
			addr += 16;
			printf("\n");
			count = 0;
		}
	}
	printf("\n");

	return 1;
}

U_BOOT_CMD(
	ddp,     3,     1,      do_dumpdpram,
	"dump dpram in pld",
	"address\n" " - dump size\n"
);

int do_setdpram(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong	addr, data;
	char *s;

	addr = simple_strtoul(argv[1], NULL, 16);
	data = simple_strtoul(argv[2], NULL, 16);

	if(addr >= 0x2000){
		printf ("Address Error\n");
		return 1;
	}

	s = (char *)0xf90002d8;
	*s = (char)addr;
	*(s + 1) = (char)(addr >> 8);

	s = (char *)0xf90002da;
	udelay(1);
	*s = (char)data;

	return 1;
}

U_BOOT_CMD(
	sdp,     3,     1,      do_setdpram,
	"set dpram in pld",
	"address\n" " - memory modify\n"
);

#define	PU_NVRAM_SYS_ADR 0x7FD0
int do_initram(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int	nbytes;
	extern char console_buffer[];
	int i;
	char *s;

	nbytes = readline (" initialize nvram (y/n) ? ");
	if (console_buffer[0] == 'y'){
		s = (char *)NVRAM_CS4_BASE;
		for(i = 0; i < CONFIG_SYS_NVRAM_SIZE - 8; i++) *s++ = 0;

		/* 23.98PsF 444 */
		s = (char *)NVRAM_CS4_BASE + PU_NVRAM_SYS_ADR;
		*s++ = 0x20;
		*s++ = 0x12;

		printf("done\n");
	}

	return 1;
}

U_BOOT_CMD(
	initram,     3,     1,      do_initram,
	"initialize nvram",
	"initram\n"
);

#define NV_HOSTNAME_SIZE 32
#define NV_DOMAIN_SIZE 60

#define NV_IP_ADDR1      0xf5007f00
#define NV_SUBNET_MASK1  0xf5007f04
#define NV_MTU_ADDR1     0xf5007f08
#define NV_GATEWAY_ADDR  0xf5007f28
#define NV_DNS_ADDR1     0xf5007f2c
#define NV_HOSTNAME      0xf5007f3c
#define NV_DOMAINNAME    0xf5007f5c
#define NV_USERID        0xf5007f98
#define NV_GROUPID       0xf5007f9a

static void show_network(void)
{
	int i, uid, gid;
	unsigned char buf[256];
	char *p;

	/* Show IP addresses  */
	p = (char *)NV_IP_ADDR1;
	for (i = 0; i < 4; i++, p += 10) {
		if(p[0] == 0) continue;
		printf("ip%u: %01d.%01d.%01d.%01d/%01d.%01d.%01d.%01d/%d\n",
			   i, p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7], (p[8] << 8) | p[9]);
	}

	/* Show Default Gateway  */
	p = (char *)NV_GATEWAY_ADDR;
	if(p[0] != 0x0)	printf("gateway: %01d.%01d.%01d.%01d\n", p[0], p[1], p[2], p[3]);

	/* Show Domain Name Server  */
	p = (char *)NV_DNS_ADDR1;
	for (i = 0; i < 4; i++, p += 4) {
		if(p[0] == 0x0) continue;
		printf("dns%u: %01d.%01d.%01d.%01d\n", i, p[0], p[1], p[2], p[3]);
	}

	/* Show hostname  */
	p = (char *)NV_HOSTNAME;
	if(p[0] != 0x00){
		for(i = 0; i < NV_HOSTNAME_SIZE; i++) buf[i] = p[i];
		buf[NV_HOSTNAME_SIZE] = 0;
		printf("hostname: %s\n", buf);
	}

	/* Show Domainname  */
	p = (char *)NV_DOMAINNAME;
	if(p[0] != 0x00){
		for(i = 0; i < NV_DOMAIN_SIZE; i++) buf[i] = p[i];
		buf[NV_DOMAIN_SIZE] = 0;
		printf("domainname: %s\n", buf);
	}

	/* Show User ID  */
	p = (char *)NV_USERID;
	uid = (p[0] << 8) | p[1];
	if(uid != 0){
		printf("UID: %d\n", uid);
	}

	/* Show Group ID  */
	p = (char *)NV_GROUPID;
	gid = (p[0] << 8) | p[1];
	if(gid != 0){
		printf("GID: %d\n", gid);
	}
}

static void setnv_def_gateway(unsigned int index, const char *string)
{
	char *p = (char *) string;
	char *dp = (char *)NV_GATEWAY_ADDR;
	unsigned int i;

	if (!string){
		for (i = 0; i < 4; i++) dp[i] = 0;
		return;
	}

	for (i = 0; *p && (i < 4); i++) {
		dp[i] = simple_strtoul(p, &p, 10);
		if (*p == '.') p++;
	}
}
static void setnv_mtu(unsigned int index, const char *string)
{
	char *p = (char *) string;
	short *dp = (short *)NV_MTU_ADDR1;

	if (index >= 4) return;
	dp += index * 5;
	if (!string){
		*dp = 0;
		return;
	}

	*dp = min(simple_strtoul(p, &p, 10), 9000);
}

static void setnv_subnet_mask(unsigned int index, const char *string)
{
	char *p = (char *) string;
	char *dp = (char *)NV_SUBNET_MASK1;
	unsigned int i;

	if (index >= 4) return;
	dp += index * 10;
	if (!string){
		for (i = 0; i < 4; i++) dp[i] = 0;
		return;
	}

	for (i = 0; *p && (i < 4); i++) {
		dp[i] = simple_strtoul(p, &p, 10);
		if (*p == '.') p++;
	}
}
static void setnv_ip_address(unsigned int index, const char *string)
{
	char *p = (char *) string;
	char *dp = (char *)NV_IP_ADDR1;
	unsigned int i;

	if (index >= 4) return;
	dp += index * 10;
	if (!string) {
		for (i = 0; i < 4; i++) dp[i] = 0;
		return;
	}

	for (i = 0; *p && (i < 4); i++) {
		dp[i] = simple_strtoul(p, &p, 10);
		if (*p == '.') p++;
	}
	if((p = strchr(p, '/')) != NULL) setnv_subnet_mask(index, ++p);
	if((p = strchr(p, '/')) != NULL) setnv_mtu(index, ++p);
}

static void setnv_dns_address(unsigned int index, const char *string)
{
	char *p = (char *) string;
	char *dp = (char *)NV_DNS_ADDR1;
	unsigned int i;

	if (index >= 4) return;
	dp += index * 4;
	if (!string){
		for (i = 0; i < 4; i++) dp[i] = 0;
		return;
	}

	for (i = 0; *p && (i < 4); i++) {
		dp[i] = simple_strtoul(p, &p, 10);
		if (*p == '.') p++;
	}
}

static void setnv_uid(unsigned int index, const char *string)
{
	char *p = (char *) string;
	short *dp = (short *)NV_USERID;

	if (!string){
		*dp = 0;
		return;
	}
	*dp = min(simple_strtoul(p, &p, 10), 0xffff);
	if(*dp < 500) *dp = 500;
}

static void setnv_gid(unsigned int index, const char *string)
{
	char *p = (char *) string;
	short *dp = (short *)NV_GROUPID;

	if (!string){
		*dp = 0;
		return;
	}
	*dp = min(simple_strtoul(p, &p, 10), 0xffff);
	if(*dp < 500) *dp = 500;
}

#define NV_CMD_IP (('i'<<8)+'p')
#define NV_CMD_SUB (('s'<<16)+('u'<<8)+'b')
#define NV_CMD_MTU (('m'<<16)+('t'<<8)+'u')
#define NV_CMD_GATE (('g'<<24)+('a'<<16)+('t'<<8)+'e')
#define NV_CMD_DNS (('d'<<16)+('n'<<8)+'s')
#define NV_CMD_HOST (('h'<<24)+('o'<<16)+('s'<<8)+'t')
#define NV_CMD_DOM (('d'<<16)+('o'<<8)+'m')
#define NV_CMD_UID (('u'<<16)+('i'<<8)+'d')
#define NV_CMD_GID (('g'<<16)+('i'<<8)+'d')

int do_net(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int i, cmd, len;
	char *p;

	if (argc == 1) {
		show_network();
		return 0;
	}

	len = strlen(argv[1]);
	if(len > 4) len = 4;

	for(i = 0, cmd = 0; i < len; i++){
		cmd <<= 8;
		cmd += argv[1][i];
	}

	switch (cmd) {
	case NV_CMD_IP:	/* ip addr */
		setnv_ip_address(argv[2][0] - '0', argv[3]);
		break;
	case NV_CMD_SUB:	/* subnet mask */
		setnv_subnet_mask(argv[2][0] - '0', argv[3]);
		break;
	case NV_CMD_MTU:	/* mtu */
		setnv_mtu(argv[2][0] - '0', argv[3]);
		break;
	case NV_CMD_GATE:	/* gateway */
		setnv_def_gateway('0', argv[2]);
		break;
	case NV_CMD_DNS:	/* domain name server */
		setnv_dns_address(argv[2][0] - '0', argv[3]);
		break;
	case NV_CMD_HOST:	/* hostname */
		p = (char *)NV_HOSTNAME;
		memset(p, 0, NV_HOSTNAME_SIZE);
		if(argv[2]) strncpy(p, argv[2], NV_HOSTNAME_SIZE);
		break;
	case NV_CMD_DOM:	/* domainname */
		p = (char *)NV_HOSTNAME;
		memset(p, 0, NV_DOMAIN_SIZE);
		if(argv[2]) strncpy(p, argv[2], NV_DOMAIN_SIZE);
		break;
	case NV_CMD_UID:	/* uid */
		setnv_uid('0', argv[2]);
		break;
	case NV_CMD_GID:	/* gid */
		setnv_gid('0', argv[2]);
		break;
	default:
		cmd_usage(cmdtp);
		break;
	}

	return 0;
}

U_BOOT_CMD(
	net, 4, 1,  do_net,
	"display and program the network settings",
	"net ip <n>|sub <n>|mtu <n>|gate|dns <n>|host|dom|uid|gid"
	"net ip <n>\n"
	"    - program the IP address for port <n>\n"
	"net sub <n>\n"
	"    - program the Subnet Mask for port <n>\n"
	"net mtu <n>\n"
	"    - program the MTU for port <n>\n"
	"net gate\n"
	"    - program the Default Gateway\n"
	"net dns <n>\n"
	"    - program the Domain Name Server\n"
	"net host\n"
	"    - program the Hostname\n"
	"net dom\n"
	"    - program the Domainname\n"
	"net uid\n"
	"    - program the User ID\n"
	"net gid\n"
	"    - program the Group ID\n"
);
