/*
 * Copyright 2009 Freescale Semiconductor, Inc.
 * (C) Copyright 2000-2006
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <watchdog.h>
#include <command.h>
#include <malloc.h>
#include <devices.h>
#include <asm/io.h>
#ifdef CONFIG_8xx
#include <mpc8xx.h>
#endif
#ifdef CONFIG_5xx
#include <mpc5xx.h>
#endif
#ifdef CONFIG_MPC5xxx
#include <mpc5xxx.h>
#endif
#if defined(CONFIG_CMD_IDE)
#include <ide.h>
#endif
#if defined(CONFIG_CMD_SCSI)
#include <scsi.h>
#endif
#if defined(CONFIG_CMD_KGDB)
#include <kgdb.h>
#endif
#ifdef CONFIG_STATUS_LED
#include <status_led.h>
#endif
#include <net.h>
#ifdef CONFIG_GENERIC_MMC
#include <mmc.h>
#endif
#include <serial.h>
#ifdef CONFIG_SYS_ALLOC_DPRAM
#if !defined(CONFIG_CPM2)
#include <commproc.h>
#endif
#endif
#include <version.h>
#if defined(CONFIG_BAB7xx)
#include <w83c553f.h>
#endif
#include <dtt.h>
#if defined(CONFIG_POST)
#include <post.h>
#endif
#if defined(CONFIG_LOGBUFFER)
#include <logbuff.h>
#endif
#if defined(CONFIG_SYS_INIT_RAM_LOCK) && defined(CONFIG_E500)
#include <asm/cache.h>
#endif
#ifdef CONFIG_PS2KBD
#include <keyboard.h>
#endif

#ifdef CONFIG_ADDR_MAP
#include <asm/mmu.h>
#endif

#ifdef CONFIG_SYS_UPDATE_FLASH_SIZE
extern int update_flash_size (int flash_size);
#endif

#if defined(CONFIG_SC3)
extern void sc3_read_eeprom(void);
#endif

#if defined(CONFIG_CMD_DOC)
void doc_init (void);
#endif
#if defined(CONFIG_HARD_I2C) || \
    defined(CONFIG_SOFT_I2C)
#include <i2c.h>
#endif
#include <spi.h>
#include <nand.h>

static char *failed = "*** failed ***\n";

#if defined(CONFIG_OXC) || defined(CONFIG_PCU_E) || defined(CONFIG_RMU)
extern flash_info_t flash_info[];
#endif

#if defined(CONFIG_START_IDE)
extern int board_start_ide(void);
#endif
#include <environment.h>
#include "myvup.h"

DECLARE_GLOBAL_DATA_PTR;

#if defined(CONFIG_ENV_IS_EMBEDDED)
#define TOTAL_MALLOC_LEN	CONFIG_SYS_MALLOC_LEN
#elif ( ((CONFIG_ENV_ADDR+CONFIG_ENV_SIZE) < CONFIG_SYS_MONITOR_BASE) || \
	(CONFIG_ENV_ADDR >= (CONFIG_SYS_MONITOR_BASE + CONFIG_SYS_MONITOR_LEN)) ) || \
      defined(CONFIG_ENV_IS_IN_NVRAM)
#define	TOTAL_MALLOC_LEN	(CONFIG_SYS_MALLOC_LEN + CONFIG_ENV_SIZE)
#else
#define	TOTAL_MALLOC_LEN	CONFIG_SYS_MALLOC_LEN
#endif

#if !defined(CONFIG_SYS_MEM_TOP_HIDE)
#define CONFIG_SYS_MEM_TOP_HIDE	0
#endif

extern ulong __init_end;
extern ulong _end;
ulong monitor_flash_len;

#if defined(CONFIG_CMD_BEDBUG)
#include <bedbug/type.h>
#endif

int pu_dpram_init(void);
int pu_dpram_check(void);
int is_pu_dpram(void);
int vup_sdram_init(char);
char vup_flash_erase(char);
char vup_flash_write(char);
int vup_send_ack(char);
int send_pu_version(void);
int send_cpu453_version(void);
unsigned short get_pu_version(int);
unsigned long get_cpu453_version(int);
int enable_pupld2_vup(void);
int disable_pupld2_vup(void);
int enable_cnpld_vup(void);
int disable_cnpupld_vup(void);
int FPLD_FlashUpdate(char, ulong, ulong);
int	CkDloaderStatus(DLOADER_CPLD_CTRL *, uint);
int config_check(void);
int read_pu_dpram(int, char *, int);
int write_pu_dpram(int, char *, int);
int fill_pu_dpram(int, char *, int);
int cpu453_dpram_init(void);
int cpu453_dpram_vup(void);
/*
 * Begin and End of memory area for malloc(), and current "brk"
 */
static	ulong	mem_malloc_start = 0;
static	ulong	mem_malloc_end	 = 0;
static	ulong	mem_malloc_brk	 = 0;

/************************************************************************
 * Utilities								*
 ************************************************************************
 */

/*
 * The Malloc area is immediately below the monitor copy in DRAM
 */
static void mem_malloc_init (void)
{
#if !defined(CONFIG_RELOC_FIXUP_WORKS)
	mem_malloc_end = CONFIG_SYS_MONITOR_BASE + gd->reloc_off;
#endif
	mem_malloc_start = mem_malloc_end - TOTAL_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;

	memset ((void *) mem_malloc_start,
		0,
		mem_malloc_end - mem_malloc_start);
}

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;

	if ((new < mem_malloc_start) || (new > mem_malloc_end)) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *) old);
}

/*
 * All attempts to come up with a "common" initialization sequence
 * that works for all boards and architectures failed: some of the
 * requirements are just _too_ different. To get rid of the resulting
 * mess of board dependend #ifdef'ed code we now make the whole
 * initialization sequence configurable to the user.
 *
 * The requirements for any new initalization function is simple: it
 * receives a pointer to the "global data" structure as it's only
 * argument, and returns an integer return code, where 0 means
 * "continue" and != 0 means "fatal error, hang the system".
 */
typedef int (init_fnc_t) (void);

/************************************************************************
 * Init Utilities							*
 ************************************************************************
 * Some of this code should be moved into the core functions,
 * but let's get it working (again) first...
 */

static int init_baudrate (void)
{
	char tmp[64];	/* long enough for environment variables */
	int i = getenv_r ("baudrate", tmp, sizeof (tmp));

	gd->baudrate = (i > 0)
			? (int) simple_strtoul (tmp, NULL, 10)
			: CONFIG_BAUDRATE;
	return (0);
}

/***********************************************************************/

void __board_add_ram_info(int use_default)
{
	/* please define platform specific board_add_ram_info() */
}
void board_add_ram_info(int) __attribute__((weak, alias("__board_add_ram_info")));


static int init_func_ram (void)
{
#ifdef	CONFIG_BOARD_TYPES
	int board_type = gd->board_type;
#else
	int board_type = 0;	/* use dummy arg */
#endif
	puts ("DRAM:  ");

	if ((gd->ram_size = initdram (board_type)) > 0) {
		print_size (gd->ram_size, "");
		board_add_ram_info(0);
		putc('\n');
		return (0);
	}
	puts (failed);
	return (1);
}

/***********************************************************************/

#if defined(CONFIG_HARD_I2C) || defined(CONFIG_SOFT_I2C)
static int init_func_i2c (void)
{
	puts ("I2C:   ");
	i2c_init (CONFIG_SYS_I2C_SPEED, CONFIG_SYS_I2C_SLAVE);
	puts ("ready\n");
	return (0);
}
#endif

#if defined(CONFIG_HARD_SPI)
static int init_func_spi (void)
{
	puts ("SPI:   ");
	spi_init ();
	puts ("ready\n");
	return (0);
}
#endif

/***********************************************************************/

#if defined(CONFIG_WATCHDOG)
static int init_func_watchdog_init (void)
{
	puts ("       Watchdog enabled\n");
	WATCHDOG_RESET ();
	return (0);
}
# define INIT_FUNC_WATCHDOG_INIT	init_func_watchdog_init,

static int init_func_watchdog_reset (void)
{
	WATCHDOG_RESET ();
	return (0);
}
# define INIT_FUNC_WATCHDOG_RESET	init_func_watchdog_reset,
#else
# define INIT_FUNC_WATCHDOG_INIT	/* undef */
# define INIT_FUNC_WATCHDOG_RESET	/* undef */
#endif /* CONFIG_WATCHDOG */

/************************************************************************
 * Initialization sequence						*
 ************************************************************************
 */

init_fnc_t *init_sequence[] = {

#if defined(CONFIG_BOARD_EARLY_INIT_F)
	board_early_init_f,
#endif
#if defined(CONFIG_MPC85xx)
	probecpu,
#endif
#if !defined(CONFIG_8xx_CPUCLK_DEFAULT)
	get_clocks,		/* get CPU and bus clocks (etc.) */
#if defined(CONFIG_TQM8xxL) && !defined(CONFIG_TQM866M) \
    && !defined(CONFIG_TQM885D)
	adjust_sdram_tbs_8xx,
#endif
	init_timebase,
#endif
#ifdef CONFIG_SYS_ALLOC_DPRAM
#if !defined(CONFIG_CPM2)
	dpram_init,
#endif
#endif
#if defined(CONFIG_BOARD_POSTCLK_INIT)
	board_postclk_init,
#endif
	env_init,
#if defined(CONFIG_8xx_CPUCLK_DEFAULT)
	get_clocks_866,		/* get CPU and bus clocks according to the environment variable */
	sdram_adjust_866,	/* adjust sdram refresh rate according to the new clock */
	init_timebase,
#endif
	init_baudrate,
	serial_init,
	console_init_f,
	display_options,
#if defined(CONFIG_8260)
	prt_8260_rsr,
	prt_8260_clks,
#endif /* CONFIG_8260 */
#if defined(CONFIG_MPC83XX)
	prt_83xx_rsr,
#endif
	checkcpu,
#if defined(CONFIG_MPC5xxx)
	prt_mpc5xxx_clks,
#endif /* CONFIG_MPC5xxx */
#if defined(CONFIG_MPC8220)
	prt_mpc8220_clks,
#endif
	checkboard,
	INIT_FUNC_WATCHDOG_INIT
#if defined(CONFIG_MISC_INIT_F)
	misc_init_f,
#endif
	INIT_FUNC_WATCHDOG_RESET
#if defined(CONFIG_HARD_I2C) || defined(CONFIG_SOFT_I2C)
	init_func_i2c,
#endif
#if defined(CONFIG_HARD_SPI)
	init_func_spi,
#endif
#ifdef CONFIG_POST
	post_init_f,
#endif
	INIT_FUNC_WATCHDOG_RESET
	init_func_ram,
#if defined(CONFIG_SYS_DRAM_TEST)
	testdram,
#endif /* CONFIG_SYS_DRAM_TEST */
	INIT_FUNC_WATCHDOG_RESET

	NULL,			/* Terminate this list */
};

ulong get_effective_memsize(void)
{
#ifndef	CONFIG_VERY_BIG_RAM
	return gd->ram_size;
#else
	/* limit stack to what we can reasonable map */
	return ((gd->ram_size > CONFIG_MAX_MEM_MAPPED) ?
		 CONFIG_MAX_MEM_MAPPED : gd->ram_size);
#endif
}

/************************************************************************
 *
 * This is the first part of the initialization sequence that is
 * implemented in C, but still running from ROM.
 *
 * The main purpose is to provide a (serial) console interface as
 * soon as possible (so we can see any error messages), and to
 * initialize the RAM so that we can relocate the monitor code to
 * RAM.
 *
 * Be aware of the restrictions: global data is read-only, BSS is not
 * initialized, and stack space is limited to a few kB.
 *
 ************************************************************************
 */

#ifdef CONFIG_LOGBUFFER
unsigned long logbuffer_base(void)
{
	return CONFIG_SYS_SDRAM_BASE + get_effective_memsize() - LOGBUFF_LEN;
}
#endif

void board_init_f (ulong bootflag)
{
	bd_t *bd;
	ulong len, addr, addr_sp;
	ulong *s;
	gd_t *id;
	init_fnc_t **init_fnc_ptr;
#ifdef CONFIG_PRAM
	int i;
	ulong reg;
	uchar tmp[64];		/* long enough for environment variables */
#endif

	/* Pointer is writable since we allocated a register for it */
	gd = (gd_t *) (CONFIG_SYS_INIT_RAM_ADDR + CONFIG_SYS_GBL_DATA_OFFSET);
	/* compiler optimization barrier needed for GCC >= 3.4 */
	__asm__ __volatile__("": : :"memory");

#if !defined(CONFIG_CPM2) && !defined(CONFIG_MPC83XX) && \
    !defined(CONFIG_MPC85xx) && !defined(CONFIG_MPC86xx)
	/* Clear initial global data */
	memset ((void *) gd, 0, sizeof (gd_t));
#endif

	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
		if ((*init_fnc_ptr) () != 0) {
			hang ();
		}
	}

	/*
	 * Now that we have DRAM mapped and working, we can
	 * relocate the code and continue running from DRAM.
	 *
	 * Reserve memory at end of RAM for (top down in that order):
	 *  - area that won't get touched by U-Boot and Linux (optional)
	 *  - kernel log buffer
	 *  - protected RAM
	 *  - LCD framebuffer
	 *  - monitor code
	 *  - board info struct
	 */
	len = (ulong)&_end - CONFIG_SYS_MONITOR_BASE;

	/*
	 * Subtract specified amount of memory to hide so that it won't
	 * get "touched" at all by U-Boot. By fixing up gd->ram_size
	 * the Linux kernel should now get passed the now "corrected"
	 * memory size and won't touch it either. This should work
	 * for arch/ppc and arch/powerpc. Only Linux board ports in
	 * arch/powerpc with bootwrapper support, that recalculate the
	 * memory size from the SDRAM controller setup will have to
	 * get fixed.
	 */
	gd->ram_size -= CONFIG_SYS_MEM_TOP_HIDE;

	addr = CONFIG_SYS_SDRAM_BASE + get_effective_memsize();

#ifdef CONFIG_LOGBUFFER
#ifndef CONFIG_ALT_LB_ADDR
	/* reserve kernel log buffer */
	addr -= (LOGBUFF_RESERVE);
	debug ("Reserving %dk for kernel logbuffer at %08lx\n", LOGBUFF_LEN, addr);
#endif
#endif

#ifdef CONFIG_PRAM
	/*
	 * reserve protected RAM
	 */
	i = getenv_r ("pram", (char *)tmp, sizeof (tmp));
	reg = (i > 0) ? simple_strtoul ((const char *)tmp, NULL, 10) : CONFIG_PRAM;
	addr -= (reg << 10);		/* size is in kB */
	debug ("Reserving %ldk for protected RAM at %08lx\n", reg, addr);
#endif /* CONFIG_PRAM */

	/* round down to next 4 kB limit */
	addr &= ~(4096 - 1);
	debug ("Top of RAM usable for U-Boot at: %08lx\n", addr);

#ifdef CONFIG_LCD
	/* reserve memory for LCD display (always full pages) */
	addr = lcd_setmem (addr);
	gd->fb_base = addr;
#endif /* CONFIG_LCD */

#if defined(CONFIG_VIDEO) && defined(CONFIG_8xx)
	/* reserve memory for video display (always full pages) */
	addr = video_setmem (addr);
	gd->fb_base = addr;
#endif /* CONFIG_VIDEO  */

	/*
	 * reserve memory for U-Boot code, data & bss
	 * round down to next 4 kB limit
	 */
	addr -= len;
	addr &= ~(4096 - 1);
#ifdef CONFIG_E500
	/* round down to next 64 kB limit so that IVPR stays aligned */
	addr &= ~(65536 - 1);
#endif

	debug ("Reserving %ldk for U-Boot at: %08lx\n", len >> 10, addr);

#ifdef CONFIG_AMIGAONEG3SE
	gd->relocaddr = addr;
#endif

	/*
	 * reserve memory for malloc() arena
	 */
	addr_sp = addr - TOTAL_MALLOC_LEN;
	debug ("Reserving %dk for malloc() at: %08lx\n",
			TOTAL_MALLOC_LEN >> 10, addr_sp);

	/*
	 * (permanently) allocate a Board Info struct
	 * and a permanent copy of the "global" data
	 */
	addr_sp -= sizeof (bd_t);
	bd = (bd_t *) addr_sp;
	gd->bd = bd;
	debug ("Reserving %zu Bytes for Board Info at: %08lx\n",
			sizeof (bd_t), addr_sp);
	addr_sp -= sizeof (gd_t);
	id = (gd_t *) addr_sp;
	debug ("Reserving %zu Bytes for Global Data at: %08lx\n",
			sizeof (gd_t), addr_sp);

	/*
	 * Finally, we set up a new (bigger) stack.
	 *
	 * Leave some safety gap for SP, force alignment on 16 byte boundary
	 * Clear initial stack frame
	 */
	addr_sp -= 16;
	addr_sp &= ~0xF;
	s = (ulong *)addr_sp;
	*s-- = 0;
	*s-- = 0;
	addr_sp = (ulong)s;
	debug ("Stack Pointer at: %08lx\n", addr_sp);

	/*
	 * Save local variables to board info struct
	 */

	bd->bi_memstart  = CONFIG_SYS_SDRAM_BASE;	/* start of  DRAM memory	*/
	bd->bi_memsize   = gd->ram_size;	/* size  of  DRAM memory in bytes */

#ifdef CONFIG_IP860
	bd->bi_sramstart = SRAM_BASE;	/* start of  SRAM memory	*/
	bd->bi_sramsize  = SRAM_SIZE;	/* size  of  SRAM memory	*/
#elif defined CONFIG_MPC8220
	bd->bi_sramstart = CONFIG_SYS_SRAM_BASE;	/* start of  SRAM memory	*/
	bd->bi_sramsize  = CONFIG_SYS_SRAM_SIZE;	/* size  of  SRAM memory	*/
#else
	bd->bi_sramstart = 0;		/* FIXME */ /* start of  SRAM memory	*/
	bd->bi_sramsize  = 0;		/* FIXME */ /* size  of  SRAM memory	*/
#endif

#if defined(CONFIG_8xx) || defined(CONFIG_8260) || defined(CONFIG_5xx) || \
    defined(CONFIG_E500) || defined(CONFIG_MPC86xx)
	bd->bi_immr_base = CONFIG_SYS_IMMR;	/* base  of IMMR register     */
#endif
#if defined(CONFIG_MPC5xxx)
	bd->bi_mbar_base = CONFIG_SYS_MBAR;	/* base of internal registers */
#endif
#if defined(CONFIG_MPC83XX)
	bd->bi_immrbar = CONFIG_SYS_IMMR;
#endif
#if defined(CONFIG_MPC8220)
	bd->bi_mbar_base = CONFIG_SYS_MBAR;	/* base of internal registers */
	bd->bi_inpfreq   = gd->inp_clk;
	bd->bi_pcifreq   = gd->pci_clk;
	bd->bi_vcofreq   = gd->vco_clk;
	bd->bi_pevfreq   = gd->pev_clk;
	bd->bi_flbfreq   = gd->flb_clk;

	/* store bootparam to sram (backward compatible), here? */
	{
		u32 *sram = (u32 *)CONFIG_SYS_SRAM_BASE;
		*sram++ = gd->ram_size;
		*sram++ = gd->bus_clk;
		*sram++ = gd->inp_clk;
		*sram++ = gd->cpu_clk;
		*sram++ = gd->vco_clk;
		*sram++ = gd->flb_clk;
		*sram++ = 0xb8c3ba11;  /* boot signature */
	}
#endif

	bd->bi_bootflags = bootflag;	/* boot / reboot flag (for LynxOS)    */

	WATCHDOG_RESET ();
	bd->bi_intfreq = gd->cpu_clk;	/* Internal Freq, in Hz */
	bd->bi_busfreq = gd->bus_clk;	/* Bus Freq,      in Hz */
#if defined(CONFIG_CPM2)
	bd->bi_cpmfreq = gd->cpm_clk;
	bd->bi_brgfreq = gd->brg_clk;
	bd->bi_sccfreq = gd->scc_clk;
	bd->bi_vco     = gd->vco_out;
#endif /* CONFIG_CPM2 */
#if defined(CONFIG_MPC512X)
	bd->bi_ipsfreq = gd->ips_clk;
#endif /* CONFIG_MPC512X */
#if defined(CONFIG_MPC5xxx)
	bd->bi_ipbfreq = gd->ipb_clk;
	bd->bi_pcifreq = gd->pci_clk;
#endif /* CONFIG_MPC5xxx */
	bd->bi_baudrate = gd->baudrate;	/* Console Baudrate     */

#ifdef CONFIG_SYS_EXTBDINFO
	strncpy ((char *)bd->bi_s_version, "1.2", sizeof (bd->bi_s_version));
	strncpy ((char *)bd->bi_r_version, U_BOOT_VERSION, sizeof (bd->bi_r_version));

	bd->bi_procfreq = gd->cpu_clk;	/* Processor Speed, In Hz */
	bd->bi_plb_busfreq = gd->bus_clk;
#if defined(CONFIG_405GP) || defined(CONFIG_405EP) || \
    defined(CONFIG_440EP) || defined(CONFIG_440GR) || \
    defined(CONFIG_440EPX) || defined(CONFIG_440GRX)
	bd->bi_pci_busfreq = get_PCI_freq ();
	bd->bi_opbfreq = get_OPB_freq ();
#elif defined(CONFIG_XILINX_405)
	bd->bi_pci_busfreq = get_PCI_freq ();
#endif
#endif

	debug ("New Stack Pointer is: %08lx\n", addr_sp);

	WATCHDOG_RESET ();

#ifdef CONFIG_POST
	post_bootmode_init();
	post_run (NULL, POST_ROM | post_bootmode_get(0));
#endif

	WATCHDOG_RESET();

	memcpy (id, (void *)gd, sizeof (gd_t));

	relocate_code (addr_sp, id, addr);

	/* NOTREACHED - relocate_code() does not return */
}

/************************************************************************
 *
 * This is the next part if the initialization sequence: we are now
 * running from RAM and have a "normal" C environment, i. e. global
 * data can be written, BSS has been cleared, the stack size in not
 * that critical any more, etc.
 *
 ************************************************************************
 */
void board_init_r (gd_t *id, ulong dest_addr)
{
	cmd_tbl_t *cmdtp;
	char *s, *e;
	bd_t *bd;
	int i;
	int pld1_init;
	extern void malloc_bin_reloc (void);
#ifndef CONFIG_ENV_IS_NOWHERE
	extern char * env_name_spec;
#endif

#ifndef CONFIG_SYS_NO_FLASH
	ulong flash_size;
#endif
	char strenv[32];

	gd = id;		/* initialize RAM version of global data */
	bd = gd->bd;

	gd->flags |= GD_FLG_RELOC;	/* tell others: relocation done */

#if defined(CONFIG_RELOC_FIXUP_WORKS)
	gd->reloc_off = 0;
	mem_malloc_end = dest_addr;
#else
	gd->reloc_off = dest_addr - CONFIG_SYS_MONITOR_BASE;
#endif

#ifdef CONFIG_SERIAL_MULTI
	serial_initialize();
#endif

#ifdef CONFIG_CW
	printf ("Now running in RAM - U-Boot at: %08lx\n", dest_addr);
#else
	debug ("Now running in RAM - U-Boot at: %08lx\n", dest_addr);
#endif

	WATCHDOG_RESET ();

	/*
	 * Setup trap handlers
	 */
	trap_init (dest_addr);

#ifdef CONFIG_ADDR_MAP
	init_addr_map();
#endif

#if defined(CONFIG_BOARD_EARLY_INIT_R)
	board_early_init_r ();
#endif

	monitor_flash_len = (ulong)&__init_end - dest_addr;

	/*
	 * We have to relocate the command table manually
	 */
	for (cmdtp = &__u_boot_cmd_start; cmdtp !=  &__u_boot_cmd_end; cmdtp++) {
		ulong addr;
		addr = (ulong) (cmdtp->cmd) + gd->reloc_off;
#if 0
		printf ("Command \"%s\": 0x%08lx => 0x%08lx\n",
				cmdtp->name, (ulong) (cmdtp->cmd), addr);
#endif
		cmdtp->cmd =
			(int (*)(struct cmd_tbl_s *, int, int, char *[]))addr;

		addr = (ulong)(cmdtp->name) + gd->reloc_off;
		cmdtp->name = (char *)addr;

		if (cmdtp->usage) {
			addr = (ulong)(cmdtp->usage) + gd->reloc_off;
			cmdtp->usage = (char *)addr;
		}
#ifdef	CONFIG_SYS_LONGHELP
		if (cmdtp->help) {
			addr = (ulong)(cmdtp->help) + gd->reloc_off;
			cmdtp->help = (char *)addr;
		}
#endif
	}
	/* there are some other pointer constants we must deal with */
#ifndef CONFIG_ENV_IS_NOWHERE
	env_name_spec += gd->reloc_off;
#endif

	WATCHDOG_RESET ();

#ifdef CONFIG_LOGBUFFER
	logbuff_init_ptrs ();
#endif
#ifdef CONFIG_POST
	post_output_backlog ();
	post_reloc ();
#endif

	WATCHDOG_RESET();

#if defined(CONFIG_SYS_DELAYED_ICACHE) || defined(CONFIG_MPC83XX)
	icache_enable ();	/* it's time to enable the instruction cache */
#endif

#if defined(CONFIG_SYS_INIT_RAM_LOCK) && defined(CONFIG_E500)
	unlock_ram_in_cache();	/* it's time to unlock D-cache in e500 */
#endif

#if defined(CONFIG_BAB7xx) || defined(CONFIG_CPC45)
	/*
	 * Do PCI configuration on BAB7xx and CPC45 _before_ the flash
	 * gets initialised, because we need the ISA resp. PCI_to_LOCAL bus
	 * bridge there.
	 */
	pci_init ();
#endif
#if defined(CONFIG_BAB7xx)
	/*
	 * Initialise the ISA bridge
	 */
	initialise_w83c553f ();
#endif

	asm ("sync ; isync");

#if !defined(CONFIG_SYS_NO_FLASH)
	puts ("FLASH: ");

	if ((flash_size = flash_init ()) > 0) {
# ifdef CONFIG_SYS_FLASH_CHECKSUM
		print_size (flash_size, "");
		/*
		 * Compute and print flash CRC if flashchecksum is set to 'y'
		 *
		 * NOTE: Maybe we should add some WATCHDOG_RESET()? XXX
		 */
		s = getenv ("flashchecksum");
		if (s && (*s == 'y')) {
			printf ("  CRC: %08X",
				crc32 (0, (const unsigned char *) CONFIG_SYS_FLASH_BASE, flash_size)
			);
		}
		putc ('\n');
# else	/* !CONFIG_SYS_FLASH_CHECKSUM */
		print_size (flash_size, "\n");
# endif /* CONFIG_SYS_FLASH_CHECKSUM */
	} else {
		puts (failed);
		hang ();
	}

	bd->bi_flashstart = CONFIG_SYS_FLASH_BASE;	/* update start of FLASH memory    */
	bd->bi_flashsize = flash_size;	/* size of FLASH memory (final value) */

#if defined(CONFIG_SYS_UPDATE_FLASH_SIZE)
	/* Make a update of the Memctrl. */
	update_flash_size (flash_size);
#endif


# if defined(CONFIG_PCU_E) || defined(CONFIG_OXC) || defined(CONFIG_RMU)
	/* flash mapped at end of memory map */
	bd->bi_flashoffset = TEXT_BASE + flash_size;
# elif CONFIG_SYS_MONITOR_BASE == CONFIG_SYS_FLASH_BASE
	bd->bi_flashoffset = monitor_flash_len;	/* reserved area for startup monitor  */
# else
	bd->bi_flashoffset = 0;
# endif
#else	/* CONFIG_SYS_NO_FLASH */

	bd->bi_flashsize = 0;
	bd->bi_flashstart = 0;
	bd->bi_flashoffset = 0;
#endif /* !CONFIG_SYS_NO_FLASH */

	WATCHDOG_RESET ();

	/* initialize higher level parts of CPU like time base and timers */
	cpu_init_r ();

	WATCHDOG_RESET ();

	/* initialize malloc() area */
	mem_malloc_init ();
	malloc_bin_reloc ();

#ifdef CONFIG_SPI
# if !defined(CONFIG_ENV_IS_IN_EEPROM)
	spi_init_f ();
# endif
	spi_init_r ();
#endif

#if defined(CONFIG_CMD_NAND)
	WATCHDOG_RESET ();
	puts ("NAND:  ");
	nand_init();		/* go init the NAND */
#endif

	/* relocate environment function pointers etc. */
	env_relocate ();

	/*
	 * Fill in missing fields of bd_info.
	 * We do this here, where we have "normal" access to the
	 * environment; we used to do this still running from ROM,
	 * where had to use getenv_r(), which can be pretty slow when
	 * the environment is in EEPROM.
	 */

#if defined(CONFIG_SYS_EXTBDINFO)
#if defined(CONFIG_405GP) || defined(CONFIG_405EP)
#if defined(CONFIG_I2CFAST)
	/*
	 * set bi_iic_fast for linux taking environment variable
	 * "i2cfast" into account
	 */
	{
		char *s = getenv ("i2cfast");
		if (s && ((*s == 'y') || (*s == 'Y'))) {
			bd->bi_iic_fast[0] = 1;
			bd->bi_iic_fast[1] = 1;
		} else {
			bd->bi_iic_fast[0] = 0;
			bd->bi_iic_fast[1] = 0;
		}
	}
#else
	bd->bi_iic_fast[0] = 0;
	bd->bi_iic_fast[1] = 0;
#endif	/* CONFIG_I2CFAST */
#endif	/* CONFIG_405GP, CONFIG_405EP */
#endif	/* CONFIG_SYS_EXTBDINFO */

#if defined(CONFIG_SC3)
	sc3_read_eeprom();
#endif

#if defined (CONFIG_ID_EEPROM) || defined (CONFIG_SYS_I2C_MAC_OFFSET)
	mac_read_from_eeprom();
#endif

	s = getenv ("ethaddr");
#if defined (CONFIG_MBX) || \
    defined (CONFIG_RPXCLASSIC) || \
    defined(CONFIG_IAD210) || \
    defined(CONFIG_V38B)
	if (s == NULL)
		board_get_enetaddr (bd->bi_enetaddr);
	else
#endif

#ifdef BOARD_CPU453
		bd->bi_enetaddr[0] = 0x3c;
		bd->bi_enetaddr[1] = 0x07;
		bd->bi_enetaddr[2] = 0x71;
		bd->bi_enetaddr[3] = 0x0f;
		bd->bi_enetaddr[4] = 0xdc;
		bd->bi_enetaddr[5] = 0xd4;
#else
		for (i = 0; i < 6; ++i) {
			bd->bi_enetaddr[i] = s ? simple_strtoul (s, &e, 16) : 0;
			if (s)
				s = (*e) ? e + 1 : e;
		}
#endif

	sprintf (strenv, "%02X:%02X:%02X:%02X:%02X:%02X",
			 bd->bi_enetaddr[0],bd->bi_enetaddr[1],bd->bi_enetaddr[2],bd->bi_enetaddr[3],bd->bi_enetaddr[4],bd->bi_enetaddr[5]);
	setenv ("ethaddr", strenv);
	printf("ethaddr     = %s\n", strenv);

#ifdef	CONFIG_HERMES
	if ((gd->board_type >> 16) == 2)
		bd->bi_ethspeed = gd->board_type & 0xFFFF;
	else
		bd->bi_ethspeed = 0xFFFF;
#endif

#ifdef CONFIG_NX823
	load_sernum_ethaddr ();
#endif

#ifdef CONFIG_HAS_ETH1
	/* handle the 2nd ethernet address */

	s = getenv ("eth1addr");

	for (i = 0; i < 6; ++i) {
		bd->bi_enet1addr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}
#endif
#ifdef CONFIG_HAS_ETH2
	/* handle the 3rd ethernet address */

	s = getenv ("eth2addr");
#if defined(CONFIG_XPEDITE1K) || defined(CONFIG_METROBOX) || defined(CONFIG_KAREF)
	if (s == NULL)
		board_get_enetaddr(bd->bi_enet2addr);
	else
#endif
	for (i = 0; i < 6; ++i) {
		bd->bi_enet2addr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}
#endif

#ifdef CONFIG_HAS_ETH3
	/* handle 4th ethernet address */
	s = getenv("eth3addr");
#if defined(CONFIG_XPEDITE1K) || defined(CONFIG_METROBOX) || defined(CONFIG_KAREF)
	if (s == NULL)
		board_get_enetaddr(bd->bi_enet3addr);
	else
#endif
	for (i = 0; i < 6; ++i) {
		bd->bi_enet3addr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}
#endif

#ifdef CONFIG_HAS_ETH4
	/* handle 5th ethernet address */
	s = getenv("eth4addr");
#if defined(CONFIG_XPEDITE1K) || defined(CONFIG_METROBOX) || defined(CONFIG_KAREF)
	if (s == NULL)
		board_get_enetaddr(bd->bi_enet4addr);
	else
#endif
	for (i = 0; i < 6; ++i) {
		bd->bi_enet4addr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}
#endif

#ifdef CONFIG_HAS_ETH5
	/* handle 6th ethernet address */
	s = getenv("eth5addr");
#if defined(CONFIG_XPEDITE1K) || defined(CONFIG_METROBOX) || defined(CONFIG_KAREF)
	if (s == NULL)
		board_get_enetaddr(bd->bi_enet5addr);
	else
#endif
	for (i = 0; i < 6; ++i) {
		bd->bi_enet5addr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}
#endif

#if defined(CONFIG_TQM8xxL) || defined(CONFIG_TQM8260) || \
    defined(CONFIG_TQM8272) || \
    defined(CONFIG_CCM) || defined(CONFIG_KUP4K) || \
    defined(CONFIG_KUP4X) || defined(CONFIG_PCS440EP)
	load_sernum_ethaddr ();
#endif

    {
		/* LED ExtPIO Initialize */
		ulong *ptr;

		ptr = (ulong *)EXTPIO_CS5_BASE;
		*(ptr + 1) = 0x00004301;
		*ptr = 0x0001fff0;
	}

#ifdef BOARD_CPU453
	{
		volatile ccsr_lbc_t *lbc = (void *)(CONFIG_SYS_MPC85xx_LBC_ADDR); // set OR6
		out_be32(&lbc->or6, 0xfff00f40);

		pld1_init =	cpu453_dpram_init();
		pld1_init =	-1;
		send_cpu453_version();
	}
#else
	{
		ulong *ptr;
		uchar *cptr;

		/* Front LED ExtPIO Initialize */
		ptr = (ulong *)PLD3_LED_CTRL;
		*ptr = 0x07070707;

		/* Fan Initializ */
		cptr = (uchar *)PLD3_FAN1_CTRL;
		*cptr = PLD3_FAN1_VAL;
		cptr = (uchar *)PLD3_FAN2_CTRL;
		*cptr = PLD3_FAN2_VAL;
		cptr = (uchar *)PLD3_FAN3_CTRL;
		*cptr = PLD3_FAN3_VAL;
		cptr = (uchar *)PLD3_FAN4_CTRL;
		*cptr = PLD3_FAN4_VAL;
		cptr = (uchar *)PLD3_FAN5_CTRL;
		*cptr = PLD3_FAN5_VAL;
			
		/* Front LED ExtPIO Initialize */
		cptr = (uchar *)PLD3_PWRLED_CTR;
		*cptr = 0x40;

		/* Default Network Mode */
		ptr = (ulong *)P2020_ERQSR_REG;
		cptr = (uchar *)DEF_NET_MODE;
		/* check eject/umount key */
		if(*ptr & 0x04000000){
			*cptr = 0x00;
		}else{
			*cptr = 0x1f;
		}

		config_check();
		pld1_init =	pu_dpram_init();
		send_pu_version();
	}
#endif

    {
#ifdef BOARD_CPU453
		bd->bi_ip_addr = 0xc0a80001; /* 192.168.0.1*/
#else
		uchar *cptr;

		/* set IP Addr */
		cptr = (uchar *)0xf5007f00;
		if(*cptr != 0){
			bd->bi_ip_addr = *cptr++;
			for(i = 0; i < 3; i++){
				bd->bi_ip_addr <<= 8;
				bd->bi_ip_addr |= *cptr++;
			}
		}else{
			bd->bi_ip_addr = 0xc0a80001; /* 192.168.0.1*/
		}
#endif
	}

	puts ("IP addr     = "); print_IPaddr(bd->bi_ip_addr); puts ("\n");
	ip_to_string(bd->bi_ip_addr, strenv);
	setenv ("ipaddr", strenv);

	WATCHDOG_RESET ();

#if defined(CONFIG_PCI) && !defined(CONFIG_BAB7xx) && !defined(CONFIG_CPC45)
	/*
	 * Do pci configuration
	 */
	pci_init ();
#endif

	{
		pci_dev_t dev;
		u16 vendor, device;
		ulong *ptr;
		char *s, *s1;
		int boot_mode;
		int bus;

		struct pci_controller *hose;

		s = (char *)EXTPIO_CS5_BASE;
		s1 = (char *)PU_PLD3_EXSW_ADDR;
		if((*s & 0x80) && (*s1 & 0x01)){
			boot_mode = 0;
		}else{
			boot_mode = 1;
		}

		if((boot_mode == 0) && (pld1_init == 0)){
			hose = pu_get_pcie1_hose();
			for(bus = hose->first_busno; bus <= hose->last_busno; bus++){
//				printf("Check bus%d\n", bus);
				dev = PCI_BDF(bus, 0x00, 0x00);
				pci_read_config_word(dev, PCI_VENDOR_ID, &vendor);
				pci_read_config_word(dev, PCI_DEVICE_ID, &device);
				if((vendor == 0x1556) || (device == 0x1100)){
//					printf("detect PLD1 PCIe. break.\n");
					break;
				}
			}

			if(bus == hose->last_busno + 1){
				printf("\nCannot detect PLD1 PCIe!!\n");

				/* ExtPIO Initialize */
				ptr = (ulong *)EXTPIO_CS5_BASE;
				*(ptr + 1) = 0x00004301;
				*ptr = 0x0001fff0;

				udelay(100);

				s = (char *)EXTPIO_CS5_BASE;
				*(s + 7) = 0x08;
				*(s + 4) = 0x01;
			}
		}
	}

/** leave this here (after malloc(), environment and PCI are working) **/
	/* Initialize devices */
	devices_init ();

	/* Initialize the jump table for applications */
	jumptable_init ();

#if defined(CONFIG_API)
	/* Initialize API */
	api_init ();
#endif

	/* Initialize the console (after the relocation and devices init) */
	console_init_r ();

#if defined(CONFIG_CCM)		|| \
    defined(CONFIG_COGENT)	|| \
    defined(CONFIG_CPCI405)	|| \
    defined(CONFIG_EVB64260)	|| \
    defined(CONFIG_KUP4K)	|| \
    defined(CONFIG_KUP4X)	|| \
    defined(CONFIG_LWMON)	|| \
    defined(CONFIG_PCU_E)	|| \
    defined(CONFIG_SC3)		|| \
    defined(CONFIG_W7O)		|| \
    defined(CONFIG_MISC_INIT_R)
	/* miscellaneous platform dependent initialisations */
	misc_init_r ();
#endif

#ifdef	CONFIG_HERMES
	if (bd->bi_ethspeed != 0xFFFF)
		hermes_start_lxt980 ((int) bd->bi_ethspeed);
#endif

#if defined(CONFIG_CMD_KGDB)
	WATCHDOG_RESET ();
	puts ("KGDB:  ");
	kgdb_init ();
#endif

	debug ("U-Boot relocated to %08lx\n", dest_addr);

	/*
	 * Enable Interrupts
	 */
	interrupt_init ();

	/* Must happen after interrupts are initialized since
	 * an irq handler gets installed
	 */
#ifdef CONFIG_SERIAL_SOFTWARE_FIFO
	serial_buffered_init();
#endif

#if defined(CONFIG_STATUS_LED) && defined(STATUS_LED_BOOT)
	status_led_set (STATUS_LED_BOOT, STATUS_LED_BLINKING);
#endif

	udelay (20);

	set_timer (0);

	/* Initialize from environment */
	if ((s = getenv ("loadaddr")) != NULL) {
		load_addr = simple_strtoul (s, NULL, 16);
	}
#if defined(CONFIG_CMD_NET)
	if ((s = getenv ("bootfile")) != NULL) {
		copy_filename (BootFile, s, sizeof (BootFile));
	}
#endif

	WATCHDOG_RESET ();

#if defined(CONFIG_DTT)		/* Digital Thermometers and Thermostats */
	dtt_init ();
#endif
#if defined(CONFIG_CMD_SCSI)
	WATCHDOG_RESET ();
	puts ("SCSI:  ");
	scsi_init ();
#endif

#ifdef CONFIG_GENERIC_MMC
	WATCHDOG_RESET ();
	puts ("MMC:  ");
	mmc_initialize (bd);
#endif

#if defined(CONFIG_CMD_DOC)
	WATCHDOG_RESET ();
	puts ("DOC:   ");
	doc_init ();
#endif

#if defined(CONFIG_CMD_NET)
#if defined(CONFIG_NET_MULTI)
	WATCHDOG_RESET ();
	puts ("Net:   ");
#endif
	eth_initialize (bd);
#endif

#if defined(CONFIG_CMD_NET) && ( \
    defined(CONFIG_CCM)		|| \
    defined(CONFIG_ELPT860)	|| \
    defined(CONFIG_EP8260)	|| \
    defined(CONFIG_IP860)	|| \
    defined(CONFIG_IVML24)	|| \
    defined(CONFIG_IVMS8)	|| \
    defined(CONFIG_MPC8260ADS)	|| \
    defined(CONFIG_MPC8266ADS)	|| \
    defined(CONFIG_MPC8560ADS)	|| \
    defined(CONFIG_PCU_E)	|| \
    defined(CONFIG_RPXSUPER)	|| \
    defined(CONFIG_STXGP3)	|| \
    defined(CONFIG_SPD823TS)	|| \
    defined(CONFIG_RESET_PHY_R)	)

	WATCHDOG_RESET ();
	debug ("Reset Ethernet PHY\n");
	reset_phy ();
#endif

#ifdef CONFIG_POST
	post_run (NULL, POST_RAM | post_bootmode_get(0));
#endif

#if defined(CONFIG_CMD_PCMCIA) \
    && !defined(CONFIG_CMD_IDE)
	WATCHDOG_RESET ();
	puts ("PCMCIA:");
	pcmcia_init ();
#endif

#if defined(CONFIG_CMD_IDE)
	WATCHDOG_RESET ();
# ifdef	CONFIG_IDE_8xx_PCCARD
	puts ("PCMCIA:");
# else
	puts ("IDE:   ");
#endif
#if defined(CONFIG_START_IDE)
	if (board_start_ide())
		ide_init ();
#else
	ide_init ();
#endif
#endif

#ifdef CONFIG_LAST_STAGE_INIT
	WATCHDOG_RESET ();
	/*
	 * Some parts can be only initialized if all others (like
	 * Interrupts) are up and running (i.e. the PC-style ISA
	 * keyboard).
	 */
	last_stage_init ();
#endif

#if defined(CONFIG_CMD_BEDBUG)
	WATCHDOG_RESET ();
	bedbug_init ();
#endif

#if defined(CONFIG_PRAM) || defined(CONFIG_LOGBUFFER)
	/*
	 * Export available size of memory for Linux,
	 * taking into account the protected RAM at top of memory
	 */
	{
		ulong pram;
		uchar memsz[32];
#ifdef CONFIG_PRAM
		char *s;

		if ((s = getenv ("pram")) != NULL) {
			pram = simple_strtoul (s, NULL, 10);
		} else {
			pram = CONFIG_PRAM;
		}
#else
		pram=0;
#endif
#ifdef CONFIG_LOGBUFFER
#ifndef CONFIG_ALT_LB_ADDR
		/* Also take the logbuffer into account (pram is in kB) */
		pram += (LOGBUFF_LEN+LOGBUFF_OVERHEAD)/1024;
#endif
#endif
		sprintf ((char *)memsz, "%ldk", (bd->bi_memsize / 1024) - pram);
		setenv ("mem", (char *)memsz);
	}
#endif

#ifdef CONFIG_PS2KBD
	puts ("PS/2:  ");
	kbd_init();
#endif

#ifdef CONFIG_MODEM_SUPPORT
 {
	 extern int do_mdm_init;
	 do_mdm_init = gd->do_mdm_init;
 }
#endif

	/* Initialization complete - start the monitor */

	/* main_loop() can return to retry autoboot, if so just run it again. */
	for (;;) {
		WATCHDOG_RESET ();
		main_loop ();
	}

	/* NOTREACHED - no way out of command loop except booting */
}

void hang (void)
{
	ulong *ptr;
	char *s;

	puts ("### ERROR ### Please RESET the board ###\n");

	/* ExtPIO Initialize */
	ptr = (ulong *)EXTPIO_CS5_BASE;
	*(ptr + 1) = 0x00004301;
	*ptr = 0x0001fff0;

	udelay(100);

	s = (char *)EXTPIO_CS5_BASE;
	*(s + 7) = 0x08;
	*(s + 4) = 0x01;

	show_boot_progress(-30);
	for (;;);
}

#ifdef CONFIG_MODEM_SUPPORT
/* called from main loop (common/main.c) */
/* 'inline' - We have to do it fast */
static inline void mdm_readline(char *buf, int bufsiz)
{
	char c;
	char *p;
	int n;

	n = 0;
	p = buf;
	for(;;) {
		c = serial_getc();

		/*		dbg("(%c)", c); */

		switch(c) {
		case '\r':
			break;
		case '\n':
			*p = '\0';
			return;

		default:
			if(n++ > bufsiz) {
				*p = '\0';
				return; /* sanity check */
			}
			*p = c;
			p++;
			break;
		}
	}
}

extern void  dbg(const char *fmt, ...);
int mdm_init (void)
{
	char env_str[16];
	char *init_str;
	int i;
	extern char console_buffer[];
	extern void enable_putc(void);
	extern int hwflow_onoff(int);

	enable_putc(); /* enable serial_putc() */

#ifdef CONFIG_HWFLOW
	init_str = getenv("mdm_flow_control");
	if (init_str && (strcmp(init_str, "rts/cts") == 0))
		hwflow_onoff (1);
	else
		hwflow_onoff(-1);
#endif

	for (i = 1;;i++) {
		sprintf(env_str, "mdm_init%d", i);
		if ((init_str = getenv(env_str)) != NULL) {
			serial_puts(init_str);
			serial_puts("\n");
			for(;;) {
				mdm_readline(console_buffer, CONFIG_SYS_CBSIZE);
				dbg("ini%d: [%s]", i, console_buffer);

				if ((strcmp(console_buffer, "OK") == 0) ||
					(strcmp(console_buffer, "ERROR") == 0)) {
					dbg("ini%d: cmd done", i);
					break;
				} else /* in case we are originating call ... */
					if (strncmp(console_buffer, "CONNECT", 7) == 0) {
						dbg("ini%d: connect", i);
						return 0;
					}
			}
		} else
			break; /* no init string - stop modem init */

		udelay(100000);
	}

	udelay(100000);

	/* final stage - wait for connect */
	for(;i > 1;) { /* if 'i' > 1 - wait for connection
				  message from modem */
		mdm_readline(console_buffer, CONFIG_SYS_CBSIZE);
		dbg("ini_f: [%s]", console_buffer);
		if (strncmp(console_buffer, "CONNECT", 7) == 0) {
			dbg("ini_f: connected");
			return 0;
		}
	}

	return 0;
}

#endif

int pu_dpram_init(void)
{
	int count;
	char result, mode, stat;
	short data;
	char *s;

	if(*(char *)PU_PLD3_SUFFIX_ADDR & 0x80){
		printf("srpc-4 mode\n");		
		mode = SETUP_SRPC_MODE_SALONE;
	}else{

		mode = *(char *)SETUP_SRPC_MODE;
		switch(mode){
		case SETUP_SRPC_MODE_SALONE:
			printf("stand alone mode\n");
			break;
		case SETUP_SRPC_MODE_VTR:
			printf("vtr mode\n");
			break;
		case SETUP_SRPC_MODE_AUTO:
		default:
			printf("auto mode\n");
			break;
		}
	}

	printf("PLD DPRAM: ");

	if(mode == SETUP_SRPC_MODE_AUTO){
		count = 0;
		while(1){
			stat = *(char *)PLD2_SRPC_STS;		
			if((stat & 0x03) == 0x03){
				break;
			}else{
				if(count++ > 1300){
					printf("no aux input ");
					mode = SETUP_SRPC_MODE_SALONE;
					break;
				}
			}

			if (tstc()) {
				printf("skip drpam check\n");
				return -1;
			}

			udelay(10000);
		}
	}

	if(mode == SETUP_SRPC_MODE_SALONE){
		if(pu_dpram_check() != 0){
			printf("verify error\n");
			return -1;
		}else{
			printf("OK\n");
			return 0;
		}
	}

	data = DPRAM_IF_CODE_BG;
	write_pu_dpram(PU_DPRAM_IFWR_ADDR, (char *)&data, 2);

	count = 0;
	while(1){

		/* SRPC mode */
		s = (char *)PLD2_SRPC_MODE;
		*s = (char)0x80;

		/* R/W buffer dir PU->MY */
		s = (char *)PLD2_DPRAM_STS;
		*s = (char)0x80;

		read_pu_dpram(PU_DPRAM_IFRD_ADDR, (char *)&data, 2);
		if(data == DPRAM_IF_CODE_BG){
			break;
		}else{
			if(count++ > 1000){
				printf("dpram veryfy error %2x, %2x\n", data, DPRAM_IF_CODE_BG);
				return -1;
			}
		}

		if (tstc()) {
			printf("skip drpam check\n");
			return -1;
		}

		udelay(10000);
	}

	if(pu_dpram_check() != 0){
		data = DPRAM_IF_CODE_ER;
		write_pu_dpram(PU_DPRAM_IFWR_ADDR, (char *)&data, 2);
		printf("verify error\n");
		return -1;
	}else{
		data = DPRAM_IF_CODE_OK;
		write_pu_dpram(PU_DPRAM_IFWR_ADDR, (char *)&data, 2);
	}

	count = 0;
	while(1){
		udelay(10000);
		read_pu_dpram(PU_DPRAM_IFRD_ADDR, (char *)&data, 2);
		if(data == DPRAM_IF_CODE_CF){
			data = DPRAM_IF_CODE_RY;
			write_pu_dpram(PU_DPRAM_IFWR_ADDR, (char *)&data, 2);
			break;
		}
		if(count++ > 1000){
			printf("dpram timeout %2x\n", DPRAM_IF_CODE_CF);
			return -1;
		}

		if (tstc()) {
			printf("skip drpam check\n");
			return -1;
		}

	}

	count = 0;
	while(1){
		udelay(10000);
		read_pu_dpram(PU_DPRAM_IFRD_ADDR, (char *)&data, 2);
		if(data == DPRAM_IF_CODE_GO){
			break;
		}
		if(count++ > 1000){
			printf("dpram timeout %2x\n", DPRAM_IF_CODE_GO);
			return -1;
		}

		if (tstc()) {
			printf("skip drpam check\n");
			return -1;
		}

	}

	result = 0x01;
	write_pu_dpram(PU_DPRAM_PU_CONNECT, &result, 1);

	printf("OK\n");
	return 0;
}

int cpu453_dpram_init(void)
{
	char *vupptr;

	printf("PLD DPRAM: ");

	vupptr = (char *)CPU453_DPRAM_ID_ADDR;

	if(*vupptr == DPRAM_VUP_CODE_CPU){
		printf("my vup sequence start!\n");
		cpu453_dpram_vup();
	}


	printf("OK\n");

	return 0;
}



char check_code_tbl[3] = {
	0x55,
	0xaa,
	0x00
};

int pu_dpram_check(void)
{
	int i, j;
	int addr;
	char buf;

	for(j = 0; j < 3; j ++){
		addr = PU_DPRAM_WR_ADDR;
		fill_pu_dpram(addr, &check_code_tbl[j], PU_DPRAM_WR_SIZE);

		udelay(10);

		addr = PU_DPRAM_WR_ADDR;
		for(i =0; i < PU_DPRAM_WR_SIZE; i++){
			read_pu_dpram(addr++, &buf, 1);
			if(buf != check_code_tbl[j]){
				printf("\n addr: %x, data: %02x/%02x\n", addr, buf, check_code_tbl[j]);
				return -1;
			}
		}
	}
	return 0;
}

int is_pu_dpram(void)
{
	short data;
	read_pu_dpram(PU_DPRAM_IFRD_ADDR, (char *)&data, 2);

	if(data != DPRAM_IF_CODE_GO) return -1;

	return 0;
}


int vup_offset_tbl[3] = {
	0x0,
	0x01000000,
	0x02000000,
};

int vup_area[3][2] = {
	{0xeff80000, 0xefffffff},
	{0xef000000, 0xef3fffff},
	{0xef400000, 0xefefffff},
};

int cpu453_dpram_vup(void)
{
	char *sptr, *dptr;
	char target, cmd, ack;
	int i, size, addr, offset, total_size;

	size = offset = addr = ack = 0;

	while(1){
		printf("check vup target\n");
		while(1){

			target = *(char *)CPU453_DPRAM_TGT_ADDR;
			if(target > 5){
				printf("vup target error\n");
				return -1;
			}
			if(target > 0){
				printf("target is %d\n", target);
				break;
			}
		}

		printf("check vup command\n");
		while(target){
			cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
			switch(cmd){
			case DPRAM_VUP_CODE_INIT:
				cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
				if(cmd != DPRAM_VUP_CODE_INIT){
					printf("cmd error %d\n", cmd);
					break;
				}
				printf("vup init\n");
				switch(target){
				case DPRAM_TARGET_BOOT:
				case DPRAM_TARGET_KERNEL:
				case DPRAM_TARGET_RMDISK:
					vup_sdram_init(target-1);
					break;
				default:
					break;
				}
				offset = vup_offset_tbl[target-1];
				vup_send_ack(VUP_OK);
				break;
			case DPRAM_VUP_CODE_TRNS:
				cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
				if(cmd != DPRAM_VUP_CODE_TRNS){
					printf("cmd error %d\n", cmd);
					break;
				}
				printf(".");
				sptr= (char *)CPU453_DPRAM_SIZE_ADDR;
				size = ((*sptr) << 8) + *(sptr +1);
				sptr = (char *)CPU453_DPRAM_DATA_ADDR;
				dptr = (char *)offset;
				for(i = 0; i < size; i++) *dptr++ = *sptr++;
				offset += size;
				vup_send_ack(VUP_OK);
				break;
			case DPRAM_VUP_CODE_ERS:
				cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
				if(cmd != DPRAM_VUP_CODE_ERS){
					printf("cmd error %d\n", cmd);
					break;
				}
				total_size = offset - vup_offset_tbl[target-1];
				switch(target){
				case DPRAM_TARGET_BOOT:
				case DPRAM_TARGET_KERNEL:
				case DPRAM_TARGET_RMDISK:
					printf("\nvup erase\n");
					ack = vup_flash_erase(target-1);
					break;
				default:
					break;
				}
				vup_send_ack(ack);
				break;
			case DPRAM_VUP_CODE_WR:
				cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
				if(cmd != DPRAM_VUP_CODE_WR){
					printf("cmd error %d\n", cmd);
					break;
				}
				total_size = offset - vup_offset_tbl[target-1];
				switch(target){
				case DPRAM_TARGET_BOOT:
				case DPRAM_TARGET_KERNEL:
				case DPRAM_TARGET_RMDISK:
					printf("\nvup write\n");
					ack = vup_flash_write(target-1);
					break;
				default:
					break;
				}
				vup_send_ack(ack);
				break;
			case DPRAM_VUP_CODE_RD:
				cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
				if(cmd != DPRAM_VUP_CODE_RD){
					printf("cmd error %d\n", cmd);
					break;
				}
				printf("*");
				sptr = (char *)CPU453_DPRAM_SIZE_ADDR;
				size = ((*sptr) << 8) + *(sptr +1);
				addr = *(int *)CPU453_DPRAM_OFST_ADDR;
				addr +=  vup_area[(int)(target - 1)][0];
				sptr = (char *)addr;
				dptr = (char *)CPU453_DPRAM_DATA_ADDR;
				for(i = 0; i < size; i++) *dptr++ = *sptr++;
				vup_send_ack(VUP_OK);
				break;
			case DPRAM_VUP_CODE_END:
				cmd = *(char *)CPU453_DPRAM_CMD_ADDR;
				if(cmd != DPRAM_VUP_CODE_END){
					printf("cmd error %d\n", cmd);
					break;
				}
				printf("\nvup finished\n");
				target = 0;
				*(char *)CPU453_DPRAM_TGT_ADDR = 0;
				vup_send_ack(VUP_OK);
				break;
			default:
				break;
			}
		}
	}
}

int vup_sdram_init(char tgt)
{
	char *ptr;
	int i;
	int size;

	ptr = (char *)vup_offset_tbl[(int)tgt];

	if(tgt < 3){
		size = vup_area[(int)tgt][1] - vup_area[(int)tgt][0] + 1;
	}else{
		size = 0x1000000;	/* pld1, pld2 temp */
	}

	debug("fill 0xff from 0x%x size 0x%x\n", (int)ptr, size);
	for(i = 0; i < size; i++) *ptr++ = 0xff;

	return 0;
}

extern flash_info_t flash_info[];
extern int do_protect (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);

char vup_flash_erase(char tgt)
{
	int rc;
	int from, to;
	int s_first[CONFIG_SYS_MAX_FLASH_BANKS], s_last[CONFIG_SYS_MAX_FLASH_BANKS], sector;
	int i, planned;
	flash_info_t *info;
	char *protect_args[4];
	cmd_tbl_t *cmdtp = NULL;

	*(char *)CPU453_DPRAM_OFST_PROG = 0;

	from = vup_area[(int)tgt][0];
	to = vup_area[(int)tgt][1];

	rc = flash_fill_sect_ranges (from, to, s_first, s_last, &planned );

	if ((planned == 0) || (rc != 0)) {
		printf("vup erase error\n");
		return VUP_ERR_ERASE;
	}

	if(tgt == 0){ /* boot */
		protect_args[0] = "protect";
		protect_args[1] = "off";
		protect_args[2] = "all";
		protect_args[3] = NULL;

		do_protect(cmdtp, 0, 3, protect_args);
	}

	debug("from: %x, to: %x, first: %x, last: %x, num: %x\n", from, to, s_first[0], s_last[0], planned);

	info = &flash_info[0];
	for(i = 0, sector = s_first[0]; sector <= s_last[0]; i++, sector++){
		rc = flash_erase (info, sector, sector);
		if (rc != 0) {
			printf("vup erase error\n");
			return VUP_ERR_ERASE;
		}
		*(char *)CPU453_DPRAM_OFST_PROG = 100 * (i + 1) / planned;
	}

//	debug("erase flash from 0x%x to 0x%x\n", from, to);
//	rc = flash_sect_erase(from, to);
//	if (rc != 0) {
//		printf("vup erase error\n");
//		return VUP_ERR_ERASE;
//	}

	*(char *)CPU453_DPRAM_OFST_PROG = 100;
	debug("done\n");

	return VUP_OK;
}

char vup_flash_write(char tgt)
{
	int rc = VUP_OK;
	int i, from, to, total, size, src, time;

	from = vup_area[(int)tgt][0];
	to = vup_area[(int)tgt][1];
	total = to - from + 1;
	src = vup_offset_tbl[(int)tgt];
	time = 0x10;
	size = total / time;

	*(char *)CPU453_DPRAM_OFST_PROG = 0;

	debug("write flash from 0x%x to 0x%x size 0x%x\n", src, from, size);

	for(i = 0; i < time; i++){
		rc = flash_write ((char *)(src + size * i), (ulong)(from + size * i), (ulong)size);
		if(rc != 0){
			flash_perror (rc);
			return VUP_ERR_WRITE;
		}
		*(char *)CPU453_DPRAM_OFST_PROG = 100 * (i + 1) / time;
	}

//	rc = flash_write ((char *)src, (ulong)from, (ulong)size);
//	if(rc != 0){
//		flash_perror (rc);
//		return VUP_ERR_WRITE;
//	}

	*(char *)CPU453_DPRAM_OFST_PROG = 100;
	printf("done\n");

	return VUP_OK;
}

int vup_send_ack(char ack)
{

	*(char *)CPU453_DPRAM_CMD_ADDR = 0;
	*(char *)CPU453_DPRAM_STS_ADDR = ack;

	return 0;
}

int ver_offset[3] = {
	0xb,
	0x2002f,
	0x3b
};

int send_pu_version(void)
{
	int i, addr;
	unsigned short ver_num;
	char buf[2];

	addr = PU_DPRAM_BOOT_VER;
	for(i = 0; i < 3; i++){
		ver_num = get_pu_version(vup_area[i][0]+ver_offset[i]);
		buf[0] = (unsigned char)ver_num;
		buf[1] = (unsigned char)(ver_num >> 8);
		write_pu_dpram(addr, buf, 2);
		addr += 2;
	}

	buf[0] = *(unsigned char *)PLD2_VERSION_ADDR;
	buf[1] = *(unsigned char *)(PLD2_VERSION_ADDR+1);
	write_pu_dpram(PU_DPRAM_PLD2_VER, buf, 2);

	return 0;
}

int cpu453_ver_offset[3] = {
	0xb,
	0x2002f,
	0x35
};

#ifdef BOARD_CPU453
int send_cpu453_version(void)
{
	int addr;

	addr = CPU453_DPRAM_BOOT_VER;
	*(unsigned short *)addr = (get_pu_version(vup_area[0][0]+cpu453_ver_offset[0]) >> 4);
	addr += 2;
	*(unsigned short *)addr = U_BOOT_SUBVERSION;
	addr += 2;
	*(unsigned long *)addr = get_cpu453_version(vup_area[1][0]+cpu453_ver_offset[1]);
	addr += 4;
	*(unsigned long *)addr = get_cpu453_version(vup_area[2][0]+cpu453_ver_offset[2]);

	return 0;
}
#endif

unsigned short get_pu_version(int address)
{
	unsigned short ver_num;
	char *ptr;

	ptr = (char *)address;
	ver_num = (simple_strtoul(ptr, NULL, 16)<<12) +
		(simple_strtoul((ptr+2), NULL, 16) & 0x0fff);

	return ver_num;
}

unsigned long get_cpu453_version(int address)
{
	unsigned long ver_num;
	char *ptr;

	ptr = (char *)address;
	ver_num = (simple_strtoul(ptr, NULL, 16) << 24) +
		(simple_strtoul((ptr+3), NULL, 16) << 16) +
		simple_strtoul((ptr+6), NULL, 16);

	return ver_num;
}

extern int pu_do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
int check_vup_mode(void)
{
	char vup_id, vup_cmd;

	vup_id = *(char *)CPU453_DPRAM_ID_ADDR;
	vup_cmd = *(char *)CPU453_DPRAM_CMD_ADDR;

	if((vup_id == DPRAM_VUP_CODE_CPU) && (vup_cmd == DPRAM_VUP_CODE_INIT)){
		printf("\n\ncpu vup request detect. reboot...\n");
		pu_do_reset (NULL, 0, 0, NULL);
	}
	return 0;
}

#define PU_GPIO_ADDR	0xffe0fc08
#define PUPLD2_RST		0x00100000
int enable_pupld2_vup(void)
{
	unsigned int *ptr;

	ptr =  (unsigned int *)PU_GPIO_ADDR;
	*ptr &= 0xffefffff;

	return 0;
}

int disable_pupld2_vup(void)
{
	unsigned int *ptr;

	ptr =  (unsigned int *)PU_GPIO_ADDR;
	*ptr |= 0x00100000;

	return 0;
}

#define CNPLD_RST		0x00200000
int enable_cnpld_vup(void)
{
	unsigned int *ptr;

	ptr =  (unsigned int *)PU_GPIO_ADDR;
	*ptr &= 0xffdfffff;

	return 0;
}

int disable_cnpld_vup(void)
{
	unsigned int *ptr;

	ptr =  (unsigned int *)PU_GPIO_ADDR;
	*ptr |= 0x00200000;

	return 0;
}

int FPLD_FlashUpdate(char dl_bank_no, ulong dl_bank_size, ulong wait)
{
	DLOADER_CPLD_CTRL *dloader = (void *)(PU_PLD_BASE +0x3FC);
	u_char	*src, data, retry_counter;
	uint	rw_adrs;

	retry_counter = 0;

	while(1){
	/* ERASE */
	dloader->command = DL_CPLD_CMD_INIT;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
		printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
		return VUP_ERR_ERASE;
	}
	dloader->data_l = dl_bank_no;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
		printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
		return VUP_ERR_ERASE;
	}
	dloader->data_h = 0x00;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
		printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
		return VUP_ERR_ERASE;
	}
	dloader->command = DL_CPLD_CMD_BANK;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
		printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
		return VUP_ERR_ERASE;
	}
	printf("ERASE...\n");
	dloader->command = DL_CPLD_CMD_ERASE;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_ERASE)) {
		if(retry_counter < 1){
			retry_counter++;
			printf("\nTimeOut!\nretry\n");
		}else{
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			return VUP_ERR_ERASE;
		}
	}else{
		break;
	}

	}
	printf("OK.\n");
	udelay(1);

	retry_counter = 0;

	while(1){
		/* WRITE */
		printf("WRITE...\n");
		dloader->command = DL_CPLD_CMD_INIT;
		udelay(1);
		if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			return VUP_ERR_WRITE;
		}
		dloader->data_l = dl_bank_no;
		udelay(1);
		if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			return VUP_ERR_WRITE;
		}
		dloader->data_h = 0x00;
		udelay(1);
		if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			return VUP_ERR_WRITE;
		}
		dloader->command = DL_CPLD_CMD_BANK;
		udelay(1);
		if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			return VUP_ERR_WRITE;
		}
		src = (uchar *)DWNLD_WORK_ADR;
		for (rw_adrs = 0; rw_adrs < dl_bank_size; rw_adrs += 2) {
			dloader->data_l = *src++;
			dloader->data_h = *src++;
			dloader->command = DL_CPLD_CMD_WRITE;
			udelay(1);
			if (CkDloaderStatus(dloader, DLOADER_WTIC_WRITE)) {
				break;
			}
		}

		if(rw_adrs >= dl_bank_size){
			break;
		}else{
			if(retry_counter < 1){
				retry_counter++;
				printf("\nTimeOut!\nretry\n");
			}else{
				printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
				printf("PLD FLASH ADDRESS=%08x\n", rw_adrs);
				return VUP_ERR_WRITE;
			}
		}
	}

	udelay(1);

	/* VERIFY */
	printf("VERIFY...\n");
	dloader->command = DL_CPLD_CMD_INIT;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
		printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
		return VUP_ERR_VERFY;
	}
	dloader->data_l = dl_bank_no;
	dloader->data_h = 0x00;
	dloader->command = DL_CPLD_CMD_BANK;
	udelay(1);
	if (CkDloaderStatus(dloader, DLOADER_WTIC_INIT)) {
		printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
		return VUP_ERR_VERFY;
	}
	src = (uchar *)DWNLD_WORK_ADR;
	for (rw_adrs = 0; rw_adrs < dl_bank_size; rw_adrs += 2) {
		data = dloader->data_l;
		udelay(1);
		if (data != *src++) {
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			printf("PLD FLASH ADDRESS=%06x(L), data=%02x(%02x)\n", rw_adrs, data, *(src-1));
			return VUP_ERR_VERFY;
		}
		data = dloader->data_h;
		udelay(1);
		if (data != *src++) {
			printf("\nV-Up ERROR %s:%d\n", __FILE__, __LINE__);
			printf("PLD FLASH ADDRESS=%06x(H), data= %02x(%02x)\n", rw_adrs, data, *(src-1));
			return VUP_ERR_VERFY;
		}
		dloader->command = DL_CPLD_CMD_READUP;
		udelay(1);
	}
	printf("OK.\n");

	dloader->command = DL_CPLD_CMD_INIT;
    return  VUP_OK;
}


int	CkDloaderStatus(DLOADER_CPLD_CTRL *dloader, uint wait_sec)
{
	uchar	st;
	uint	start_tic;
	uint	start_sec;

	start_tic = 0;
	start_sec = 0;

	do {
		st = dloader->status;
		st &= 0x1e; 	/* b4->b1 */
		if (st == (DL_CPLD_STS_E_IDLE | DL_CPLD_STS_W_IDLE | DL_CPLD_STS_BK_IDLE)) {
//			printf("OK, cmd=%x, time=%04x\n",dloader->command, start_tic);
			return 0;
		}

		if(start_tic++ > DLOADER_WTIC_1SEC){
			start_sec++;
			start_tic = 0;
//			printf("status=%x\n", dloader->status);
		}
	} while (start_sec < wait_sec);

	printf("NG, cmd=%02x, status=%02x, time=%02dsec\n",
		   dloader->command, dloader->status, wait_sec);
	return 1;
}

int config_check(void)
{
	volatile uchar *ppio;
	volatile uchar *ppld;
	int i;

	ppio = (uchar *)(EXTPIO_CS5_BASE + 3);

	for(i = 0; i < 400; i++){
		if(*ppio & 0x04){
			printf("FPGA config done\n");
			return 0;
		}
		udelay(10000);
	}

	printf("FPGA reconfig start\n");

	ppld = (uchar *)PLD2_CONFIG_REQ;
	*ppld = PLD2_CONFIG_INIT;
	udelay(100);
	*ppld = PLD2_CONFIG_BOOT;

	for(i = 0; i < 500; i++){
		if(*ppio & 0x04){
			printf("FPGA reconfig done\n");
			*ppld = PLD2_CONFIG_INIT;
			return 0;
		}
		udelay(10000);
	}

	*ppld = PLD2_CONFIG_INIT;
	printf("FPGA config timeout\n");

	return 1;
}

int read_pu_dpram(int addr, char *buf, int size)
{
	int i;
	char *s;

	s = (char *)PLD2_DPRAM_LA;

	*s = (char)addr;
	*(s + 1) = (char)(addr >> 8);

	s = (char *)PLD2_DPRAM_RD;
	for(i = 0; i < size; i++){
		udelay(1);
		*buf++ = *s;
	}

	return 1;
}

int write_pu_dpram(int addr, char *buf, int size)
{
	int i;
	char *s;

	s = (char *)PLD2_DPRAM_LA;

	*s = (char)addr;
	*(s + 1) = (char)(addr >> 8);

	s = (char *)PLD2_DPRAM_WD;
	for(i = 0; i < size; i++){
		udelay(1);
		*s = *buf++;
	}

	return 1;
}

int fill_pu_dpram(int addr, char *buf, int size)
{
	int i;
	char *s;

	s = (char *)PLD2_DPRAM_LA;

	*s = (char)addr;
	*(s + 1) = (char)(addr >> 8);

	s = (char *)PLD2_DPRAM_WD;
	for(i = 0; i < size; i++) *s = *buf;

	return 1;
}

#if 0 /* We could use plain global data, but the resulting code is bigger */
/*
 * Pointer to initial global data area
 *
 * Here we initialize it.
 */
#undef	XTRN_DECLARE_GLOBAL_DATA_PTR
#define XTRN_DECLARE_GLOBAL_DATA_PTR	/* empty = allocate here */
DECLARE_GLOBAL_DATA_PTR = (gd_t *) (CONFIG_SYS_INIT_RAM_ADDR + CONFIG_SYS_GBL_DATA_OFFSET);
#endif  /* 0 */

/************************************************************************/
