/*------------------------------------------------
 myvup.h
        Header File of MY Version Upgrade
------------------------------------------------*/
#ifndef MYVUP_H_DEF
#define MYVUP_H_DEF

#define P2020_ERQSR_REG 0xffe41308

#define SETUP_SRPC_MODE 0xf5001001
#define DEF_NET_MODE    0xf5007ff0

#define PLD2_SRPC_MODE 0xf9000270
#define PLD2_DPRAM_STS 0xf9000271
#define PLD2_SRPC_STS  0xf9000272

#define PLD2_DPRAM_LA 0xf90002d8
#define PLD2_DPRAM_UA 0xf90002d9
#define PLD2_DPRAM_WD 0xf90002da
#define PLD2_DPRAM_RD 0xf90002db

#define PLD2_VERSION_ADDR    0xf900000e
#define PLD2_CONFIG_REQ      0xf90003fe

#define PLD2_CONFIG_INIT     0x00
#define PLD2_CONFIG_BOOT     0x04

#define PLD3_BOARD_SUFIX     0xf9008011
#define PLD3_VERSION_ADDR    0xf9008012
#define PLD3_DIP_SWITCH      0xf9008020
#define PLD3_PWRLED_CTR      0xf9008031
#define PLD3_LED_CTRL        0xf9008040
#define PLD3_LED_CTRL        0xf9008040
#define PLD3_FAN1_CTRL		 0xf9008061
#define PLD3_FAN2_CTRL		 0xf9008062
#define PLD3_FAN3_CTRL		 0xf9008063
#define PLD3_FAN4_CTRL		 0xf9008064
#define PLD3_FAN5_CTRL		 0xf9008065

#define PLD3_FAN1_VAL (0x60)
#define PLD3_FAN2_VAL (0x4A)
#define PLD3_FAN3_VAL (0x40)
#define PLD3_FAN4_VAL (0x55)
#define PLD3_FAN5_VAL (0x4A)

#define PLD3_LED_BLUE        0x01
#define PLD3_LED_GRN         0x02
#define PLD3_LED_RED         0x04
#define PLD3_LED_BLINK       0x08

#define PU_DPRAM_IP_ADDR     0x0f0
#define PU_DPRAM_SUBNET_MSK  0x0f4
#define PU_DPRAM_DEF_GATEWAY 0x0f8

#define PU_DPRAM_BOOT_VER  0x0f0c
#define PU_DPRAM_KERNL_VER 0x0f0e
#define PU_DPRAM_RDSK_VER  0x0f10
#define PU_DPRAM_PLD1_VER  0x0f12
#define PU_DPRAM_PLD2_VER  0x0f14

#define PU_DPRAM_IFRD_ADDR 0x07fe

#define PU_DPRAM_PU_MODE   0x0880
#define PU_DPRAM_PU_CONNECT 0x0881
#define PU_DPRAM_ACK_ADDR  0x0ff0
#define PU_DPRAM_IFWR_ADDR 0x0ffe

#define PU_DPRAM_WR_ADDR   0x0800
#define PU_DPRAM_DATA_ADDR 0x1000

#define PU_DPRAM_SIZE 0x2000
#define PU_DPRAM_WR_SIZE 0x800

#define DPRAM_IF_CODE_BG	(0x4742)
#define DPRAM_IF_CODE_OK	(0x4b4f)
#define DPRAM_IF_CODE_ER	(0x5245)
#define DPRAM_IF_CODE_CF	(0x4643)
#define DPRAM_IF_CODE_RY	(0x5952)
#define DPRAM_IF_CODE_GO	(0x4f47)

#define DPRAM_VUP_CODE_MY	(0x0a)
#define DPRAM_VUP_CODE_CPU	(0x0b)
#define DPRAM_VUP_CODE_INIT	(0x01)
#define DPRAM_VUP_CODE_TRNS	(0x02)
#define DPRAM_VUP_CODE_ERS	(0x03)
#define DPRAM_VUP_CODE_WR	(0x04)
#define DPRAM_VUP_CODE_RD	(0x05)
#define DPRAM_VUP_CODE_END	(0x06)

#define DPRAM_TARGET_BOOT	(0x01)
#define DPRAM_TARGET_KERNEL	(0x02)
#define DPRAM_TARGET_RMDISK	(0x03)

#define VUP_OK        1
#define VUP_ERR_ERASE 2
#define VUP_ERR_WRITE 3
#define VUP_ERR_VERFY 4


#define SETUP_SRPC_MODE_AUTO   0
#define SETUP_SRPC_MODE_SALONE 1
#define SETUP_SRPC_MODE_VTR    2

typedef struct  _DLOADER_CPLD_CTRL {
    volatile unsigned char   data_l;   /* +00 Data Register [7:0] */
    volatile unsigned char   data_h;   /* +01 Data Register [15:0] */
    volatile unsigned char   command;  /* +02 Command Register [2:0] */
    volatile unsigned char   status;   /* +03 Status Register [5:0] */
}   DLOADER_CPLD_CTRL;

#define DL_CPLD_BANK_ALL        0x02        /* ALL AREA                       */
#define DL_CPLD_CMD_INIT        0x00        /* COMMAND: Init                  */
#define DL_CPLD_CMD_ERASE       0x01        /* COMMAND: Erase                 */
#define DL_CPLD_CMD_WRITE       0x02        /* COMMAND: Write                 */
#define DL_CPLD_CMD_BANK        0x03        /* COMMAND: Bank                  */
#define DL_CPLD_CMD_BOOT        0x04        /* COMMAND: Boot                  */
#define DL_CPLD_CMD_READUP      0x0f        /* COMMAND: Read Address Up       */
#define DL_CPLD_STS_B_IDLE      0x01        /* STATUS: Boot Idle              */
#define DL_CPLD_STS_E_IDLE      0x02        /* STATUS: Erase Idle             */
#define DL_CPLD_STS_W_IDLE      0x04        /* STATUS: Write Idle             */
#define DL_CPLD_STS_BK_IDLE     0x08        /* STATUS: Bank Idle              */
#define DL_CPLD_STS_BUSY        0x10        /* STATUS: Flash BUSY             */
#define	DLOADER_WTIC_INIT	(1)
#define	DLOADER_WTIC_ERASE	(300)
#define	DLOADER_WTIC_WRITE	(1)
#define	DLOADER_WTIC_1SEC	(0x388000)	/* 8540 750M not acuurate */

#define DWNLD_WORK_ADR 0x10000000
#define DWNLD_WORK_SIZE 0x01000000

/* for CPU453 */
#define CPU453_DPRAM_BOOT_VER  0x0f9006fe0
#define CPU453_DPRAM_ID_ADDR   0x0f9006ff0
#define CPU453_DPRAM_TGT_ADDR  0x0f9006ff1
#define CPU453_DPRAM_CMD_ADDR  0x0f9006ff2
#define CPU453_DPRAM_STS_ADDR  0x0f9006ff3
#define CPU453_DPRAM_SIZE_ADDR 0x0f9006ff4
#define CPU453_DPRAM_OFST_PROG 0x0f9006ff6
#define CPU453_DPRAM_OFST_ADDR 0x0f9006ff8
#define CPU453_DPRAM_DATA_ADDR 0x0f9007000

extern int enable_pupld2_vup(void);
extern int disable_pupld2_vup(void);
extern int Fpga_Update(long);
extern int FPLD_FlashUpdate(char, unsigned long, unsigned long);
extern int read_pu_dpram(int, char *, int);
extern int write_pu_dpram(int, char *, int);
extern int fill_pu_dpram(int, char *, int);

#endif
